@@ -12,6 +12,7 @@
 #include "nsAString.h"
 #include "nsComponentManagerUtils.h"    // for do_CreateInstance
 #include "nsComposerCommandsUpdater.h"  // for nsComposerCommandsUpdater
+#include "nsContentUtils.h"
 #include "nsDebug.h"                    // for NS_ENSURE_SUCCESS, etc
 #include "nsEditingSession.h"
 #include "nsError.h"                    // for NS_ERROR_FAILURE, NS_OK, etc
@@ -992,7 +993,8 @@ nsEditingSession::TimerCallback(nsITimer* aTimer, void* aClosure)
   if (docShell) {
     nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(docShell));
     if (webNav) {
-      webNav->LoadURI(u"about:blank", 0, nullptr, nullptr, nullptr);
+      webNav->LoadURI(u"about:blank", 0, nullptr, nullptr, nullptr,
+                      nsContentUtils::GetSystemPrincipal());
     }
   }
 }
