@@ -8,20 +8,22 @@
 use Atom;
 use bezier::Bezier;
 use context::SharedStyleContext;
-use dom::{OpaqueNode, UnsafeNode};
+use dom::OpaqueNode;
 use euclid::point::Point2D;
-use keyframes::{KeyframesStep, KeyframesStepValue};
+use font_metrics::FontMetricsProvider;
 use properties::{self, CascadeFlags, ComputedValues, Importance};
 use properties::animated_properties::{AnimatedProperty, TransitionProperty};
 use properties::longhands::animation_direction::computed_value::single_value::T as AnimationDirection;
 use properties::longhands::animation_iteration_count::single_value::computed_value::T as AnimationIterationCount;
 use properties::longhands::animation_play_state::computed_value::single_value::T as AnimationPlayState;
-use properties::longhands::transition_timing_function::single_value::computed_value::StartEnd;
-use properties::longhands::transition_timing_function::single_value::computed_value::T as TransitionTimingFunction;
-use std::sync::Arc;
+use rule_tree::CascadeLevel;
 use std::sync::mpsc::Sender;
+use stylearc::Arc;
+use stylesheets::keyframes_rule::{KeyframesStep, KeyframesStepValue};
 use timer::Timer;
 use values::computed::Time;
+use values::computed::transform::TimingFunction;
+use values::generics::transform::{StepPosition, TimingFunction as GenericTimingFunction};
 
 /// This structure represents a keyframes animation current iteration state.
 ///
@@ -187,7 +189,7 @@ pub enum Animation {
     /// the f64 field is the start time as returned by `time::precise_time_s()`.
     ///
     /// The `bool` field is werther this animation should no longer run.
-    Transition(OpaqueNode, UnsafeNode, f64, AnimationFrame, bool),
+    Transition(OpaqueNode, f64, AnimationFrame, bool),
     /// A keyframes animation is identified by a name, and can have a
     /// node-dependent state (i.e. iteration count, etc.).
     Keyframes(OpaqueNode, Atom, KeyframesAnimationState),
@@ -199,7 +201,7 @@ impl Animation {
     pub fn mark_as_expired(&mut self) {
         debug_assert!(!self.is_expired());
         match *self {
-            Animation::Transition(_, _, _, _, ref mut expired) => *expired = true,
+            Animation::Transition(_, _, _, ref mut expired) => *expired = true,
             Animation::Keyframes(_, _, ref mut state) => state.expired = true,
         }
     }
@@ -208,7 +210,7 @@ impl Animation {
     #[inline]
     pub fn is_expired(&self) -> bool {
         match *self {
-            Animation::Transition(_, _, _, _, expired) => expired,
+            Animation::Transition(_, _, _, expired) => expired,
             Animation::Keyframes(_, _, ref state) => state.expired,
         }
     }
@@ -217,7 +219,7 @@ impl Animation {
     #[inline]
     pub fn node(&self) -> &OpaqueNode {
         match *self {
-            Animation::Transition(ref node, _, _, _, _) => node,
+            Animation::Transition(ref node, _, _, _) => node,
             Animation::Keyframes(ref node, _, _) => node,
         }
     }
@@ -230,6 +232,15 @@ impl Animation {
             Animation::Keyframes(_, _, ref state) => state.is_paused(),
         }
     }
+
+    /// Whether this animation is a transition.
+    #[inline]
+    pub fn is_transition(&self) -> bool {
+        match *self {
+            Animation::Transition(..) => true,
+            Animation::Keyframes(..) => false,
+        }
+    }
 }
 
 
@@ -247,7 +258,7 @@ pub struct AnimationFrame {
 #[derive(Debug, Clone)]
 pub struct PropertyAnimation {
     property: AnimatedProperty,
-    timing_function: TransitionTimingFunction,
+    timing_function: TimingFunction,
     duration: Time, // TODO: isn't this just repeated?
 }
 
@@ -271,10 +282,23 @@ impl PropertyAnimation {
         let timing_function = box_style.transition_timing_function_mod(transition_index);
         let duration = box_style.transition_duration_mod(transition_index);
 
+        if let TransitionProperty::Unsupported(_) = transition_property {
+            return result
+        }
+
+        if transition_property.is_shorthand() {
+            return transition_property.longhands().iter().filter_map(|transition_property| {
+                PropertyAnimation::from_transition_property(transition_property,
+                                                            timing_function,
+                                                            duration,
+                                                            old_style,
+                                                            new_style)
+            }).collect();
+        }
 
         if transition_property != TransitionProperty::All {
             if let Some(property_animation) =
-                    PropertyAnimation::from_transition_property(transition_property,
+                    PropertyAnimation::from_transition_property(&transition_property,
                                                                 timing_function,
                                                                 duration,
                                                                 old_style,
@@ -286,7 +310,7 @@ impl PropertyAnimation {
 
         TransitionProperty::each(|transition_property| {
             if let Some(property_animation) =
-                    PropertyAnimation::from_transition_property(transition_property,
+                    PropertyAnimation::from_transition_property(&transition_property,
                                                                 timing_function,
                                                                 duration,
                                                                 old_style,
@@ -298,13 +322,15 @@ impl PropertyAnimation {
         result
     }
 
-    fn from_transition_property(transition_property: TransitionProperty,
-                                timing_function: TransitionTimingFunction,
+    fn from_transition_property(transition_property: &TransitionProperty,
+                                timing_function: TimingFunction,
                                 duration: Time,
                                 old_style: &ComputedValues,
                                 new_style: &ComputedValues)
                                 -> Option<PropertyAnimation> {
-        let animated_property = AnimatedProperty::from_transition_property(&transition_property,
+        debug_assert!(!transition_property.is_shorthand() &&
+                      transition_property != &TransitionProperty::All);
+        let animated_property = AnimatedProperty::from_transition_property(transition_property,
                                                                            old_style,
                                                                            new_style);
 
@@ -323,19 +349,44 @@ impl PropertyAnimation {
 
     /// Update the given animation at a given point of progress.
     pub fn update(&self, style: &mut ComputedValues, time: f64) {
+        let solve_bezier = |(p1, p2): (Point2D<_>, Point2D<_>)| {
+            let epsilon = 1. / (200. * (self.duration.seconds() as f64));
+            let bezier = Bezier::new(
+                Point2D::new(p1.x as f64, p1.y as f64),
+                Point2D::new(p2.x as f64, p2.y as f64),
+            );
+            bezier.solve(time, epsilon)
+        };
+
         let progress = match self.timing_function {
-            TransitionTimingFunction::CubicBezier(p1, p2) => {
-                // See `WebCore::AnimationBase::solveEpsilon(double)` in WebKit.
-                let epsilon = 1.0 / (200.0 * (self.duration.seconds() as f64));
-                Bezier::new(Point2D::new(p1.x as f64, p1.y as f64),
-                            Point2D::new(p2.x as f64, p2.y as f64)).solve(time, epsilon)
-            }
-            TransitionTimingFunction::Steps(steps, StartEnd::Start) => {
+            GenericTimingFunction::CubicBezier(p1, p2) => {
+                solve_bezier((p1, p2))
+            },
+            GenericTimingFunction::Steps(steps, StepPosition::Start) => {
                 (time * (steps as f64)).ceil() / (steps as f64)
-            }
-            TransitionTimingFunction::Steps(steps, StartEnd::End) => {
+            },
+            GenericTimingFunction::Steps(steps, StepPosition::End) => {
                 (time * (steps as f64)).floor() / (steps as f64)
-            }
+            },
+            GenericTimingFunction::Frames(frames) => {
+                // https://drafts.csswg.org/css-timing/#frames-timing-functions
+                let mut out = (time * (frames as f64)).floor() / ((frames - 1) as f64);
+                if out > 1.0 {
+                    // FIXME: Basically, during the animation sampling process, the input progress
+                    // should be in the range of [0, 1]. However, |time| is not accurate enough
+                    // here, which means |time| could be larger than 1.0 in the last animation
+                    // frame. (It should be equal to 1.0 exactly.) This makes the output of frames
+                    // timing function jumps to the next frame/level.
+                    // However, this solution is still not correct because |time| is possible
+                    // outside the range of [0, 1] after introducing Web Animations. We should fix
+                    // this problem when implementing web animations.
+                    out = 1.0;
+                }
+                out
+            },
+            GenericTimingFunction::Keyword(keyword) => {
+                solve_bezier(keyword.to_bezier_points())
+            },
         };
 
         self.property.update(style, progress);
@@ -343,7 +394,7 @@ impl PropertyAnimation {
 
     #[inline]
     fn does_animate(&self) -> bool {
-        self.property.does_animate() && self.duration != Time(0.0)
+        self.property.does_animate() && self.duration.seconds() != 0.0
     }
 
     /// Whether this animation has the same end value as another one.
@@ -359,9 +410,9 @@ impl PropertyAnimation {
 //
 // TODO(emilio): Take rid of this mutex splitting SharedLayoutContex into a
 // cloneable part and a non-cloneable part..
+#[cfg(feature = "servo")]
 pub fn start_transitions_if_applicable(new_animations_sender: &Sender<Animation>,
                                        opaque_node: OpaqueNode,
-                                       unsafe_node: UnsafeNode,
                                        old_style: &ComputedValues,
                                        new_style: &mut Arc<ComputedValues>,
                                        timer: &Timer,
@@ -380,7 +431,7 @@ pub fn start_transitions_if_applicable(new_animations_sender: &Sender<Animation>
             // [1]: https://drafts.csswg.org/css-transitions/#starting
             if possibly_expired_animations.iter().any(|animation| {
                     animation.has_the_same_end_value_as(&property_animation)
-               }) {
+                }) {
                 continue
             }
 
@@ -395,7 +446,7 @@ pub fn start_transitions_if_applicable(new_animations_sender: &Sender<Animation>
             let start_time =
                 now + (box_style.transition_delay_mod(i).seconds() as f64);
             new_animations_sender
-                .send(Animation::Transition(opaque_node, unsafe_node, start_time, AnimationFrame {
+                .send(Animation::Transition(opaque_node, start_time, AnimationFrame {
                     duration: box_style.transition_duration_mod(i).seconds() as f64,
                     property_animation: property_animation,
                 }, /* is_expired = */ false)).unwrap();
@@ -410,32 +461,37 @@ pub fn start_transitions_if_applicable(new_animations_sender: &Sender<Animation>
 fn compute_style_for_animation_step(context: &SharedStyleContext,
                                     step: &KeyframesStep,
                                     previous_style: &ComputedValues,
-                                    style_from_cascade: &ComputedValues)
+                                    style_from_cascade: &ComputedValues,
+                                    font_metrics_provider: &FontMetricsProvider)
                                     -> ComputedValues {
     match step.value {
         KeyframesStepValue::ComputedValues => style_from_cascade.clone(),
         KeyframesStepValue::Declarations { block: ref declarations } => {
-            let guard = declarations.read();
+            let guard = declarations.read_with(context.guards.author);
 
             // No !important in keyframes.
-            debug_assert!(guard.declarations.iter()
+            debug_assert!(guard.declarations().iter()
                             .all(|&(_, importance)| importance == Importance::Normal));
 
             let iter = || {
-                guard.declarations.iter().rev().map(|&(ref decl, _importance)| decl)
+                guard.declarations().iter().rev()
+                     .map(|&(ref decl, _importance)| (decl, CascadeLevel::Animations))
             };
 
+            // This currently ignores visited styles, which seems acceptable,
+            // as existing browsers don't appear to animate visited styles.
             let computed =
-                properties::apply_declarations(context.viewport_size,
+                properties::apply_declarations(context.stylist.device(),
                                                /* is_root = */ false,
                                                iter,
                                                previous_style,
                                                previous_style,
-                                               &context.default_computed_values,
                                                /* cascade_info = */ None,
-                                               context.error_reporter.clone(),
-                                               /* Metrics provider */ None,
-                                               CascadeFlags::empty());
+                                               /* visited_style = */ None,
+                                               &*context.error_reporter,
+                                               font_metrics_provider,
+                                               CascadeFlags::empty(),
+                                               context.quirks_mode);
             computed
         }
     }
@@ -452,13 +508,19 @@ pub fn maybe_start_animations(context: &SharedStyleContext,
 
     let box_style = new_style.get_box();
     for (i, name) in box_style.animation_name_iter().enumerate() {
+        let name = if let Some(atom) = name.as_atom() {
+            atom
+        } else {
+            continue
+        };
+
         debug!("maybe_start_animations: name={}", name);
         let total_duration = box_style.animation_duration_mod(i).seconds();
         if total_duration == 0. {
             continue
         }
 
-        if let Some(ref anim) = context.stylist.animations().get(&name.0) {
+        if let Some(ref anim) = context.stylist.animations().get(name) {
             debug!("maybe_start_animations: animation {} found", name);
 
             // If this animation doesn't have any keyframe, we can just continue
@@ -494,7 +556,7 @@ pub fn maybe_start_animations(context: &SharedStyleContext,
 
 
             new_animations_sender
-                .send(Animation::Keyframes(node, name.0.clone(), KeyframesAnimationState {
+                .send(Animation::Keyframes(node, name.clone(), KeyframesAnimationState {
                     started_at: animation_start,
                     duration: duration as f64,
                     delay: delay as f64,
@@ -535,12 +597,13 @@ pub fn update_style_for_animation_frame(mut new_style: &mut Arc<ComputedValues>,
 /// If `damage` is provided, inserts the appropriate restyle damage.
 pub fn update_style_for_animation(context: &SharedStyleContext,
                                   animation: &Animation,
-                                  style: &mut Arc<ComputedValues>) {
+                                  style: &mut Arc<ComputedValues>,
+                                  font_metrics_provider: &FontMetricsProvider) {
     debug!("update_style_for_animation: entering");
     debug_assert!(!animation.is_expired());
 
     match *animation {
-        Animation::Transition(_, _, start_time, ref frame, _) => {
+        Animation::Transition(_, start_time, ref frame, _) => {
             debug!("update_style_for_animation: transition found");
             let now = context.timer.seconds();
             let mut new_style = (*style).clone();
@@ -571,9 +634,10 @@ pub fn update_style_for_animation(context: &SharedStyleContext,
 
             debug_assert!(!animation.steps.is_empty());
 
-            let maybe_index = style.get_box()
-                                   .animation_name_iter()
-                                   .position(|animation_name| *name == animation_name.0);
+            let maybe_index = style
+                .get_box()
+                .animation_name_iter()
+                .position(|animation_name| Some(name) == animation_name.as_atom());
 
             let index = match maybe_index {
                 Some(index) => index,
@@ -659,7 +723,8 @@ pub fn update_style_for_animation(context: &SharedStyleContext,
             let from_style = compute_style_for_animation_step(context,
                                                               last_keyframe,
                                                               &**style,
-                                                              &state.cascade_style);
+                                                              &state.cascade_style,
+                                                              font_metrics_provider);
 
             // NB: The spec says that the timing function can be overwritten
             // from the keyframe style.
@@ -673,16 +738,17 @@ pub fn update_style_for_animation(context: &SharedStyleContext,
             let target_style = compute_style_for_animation_step(context,
                                                                 target_keyframe,
                                                                 &from_style,
-                                                                &state.cascade_style);
+                                                                &state.cascade_style,
+                                                                font_metrics_provider);
 
             let mut new_style = (*style).clone();
 
             for transition_property in &animation.properties_changed {
                 debug!("update_style_for_animation: scanning prop {:?} for animation \"{}\"",
                        transition_property, name);
-                match PropertyAnimation::from_transition_property(*transition_property,
+                match PropertyAnimation::from_transition_property(transition_property,
                                                                   timing_function,
-                                                                  Time(relative_duration as f32),
+                                                                  Time::from_seconds(relative_duration as f32),
                                                                   &from_style,
                                                                   &target_style) {
                     Some(property_animation) => {
@@ -704,6 +770,7 @@ pub fn update_style_for_animation(context: &SharedStyleContext,
 }
 
 /// Update the style in the node when it finishes.
+#[cfg(feature = "servo")]
 pub fn complete_expired_transitions(node: OpaqueNode, style: &mut Arc<ComputedValues>,
                                     context: &SharedStyleContext) -> bool {
     let had_animations_to_expire;
@@ -714,7 +781,7 @@ pub fn complete_expired_transitions(node: OpaqueNode, style: &mut Arc<ComputedVa
         if let Some(ref animations) = animations_to_expire {
             for animation in *animations {
                 // TODO: support animation-fill-mode
-                if let Animation::Transition(_, _, _, ref frame, _) = *animation {
+                if let Animation::Transition(_, _, ref frame, _) = *animation {
                     frame.property_animation.update(Arc::make_mut(style), 1.0);
                 }
             }
