@@ -5,8 +5,10 @@
 //! Rust sugar and convenience methods for Gecko types.
 
 mod ns_com_ptr;
+mod ns_compatibility;
 mod ns_css_shadow_array;
-mod ns_css_value;
+mod ns_css_shadow_item;
+pub mod ns_css_value;
 mod ns_style_auto_array;
 pub mod ns_style_coord;
 mod ns_t_array;
