@@ -5,7 +5,6 @@
 from __future__ import absolute_import, unicode_literals
 
 import logging
-import os
 
 from . import (
     target_tasks,
@@ -13,8 +12,6 @@ from . import (
 
 logger = logging.getLogger(__name__)
 
-GECKO = os.path.realpath(os.path.join(os.path.dirname(__file__), '..', '..'))
-
 filter_task_functions = {}
 
 
@@ -40,20 +37,17 @@ def filter_target_tasks(graph, parameters):
 
 @filter_task('check_servo')
 def filter_servo(graph, parameters):
-    """Filters out tasks requiring Servo if Servo isn't present."""
-    if os.path.exists(os.path.join(GECKO, 'servo', 'components', 'style')):
-        return graph.tasks.keys()
-
-    logger.info('servo/ directory not present; removing tasks requiring it')
+    """bug 1339542 When Servo vcs sync service lands Servo commits in
+    autoland repo, run linux64-stylo tests but skip other
+    platforms (to  reduce test load)."""
 
     SERVO_PLATFORMS = {
         'linux64-stylo',
     }
 
     def fltr(task):
-        if task.attributes.get('build_platform') in SERVO_PLATFORMS:
-            return False
-
-        return True
+        if parameters.get('owner') != "servo-vcs-sync@mozilla.com":
+            return True
+        return task.attributes.get('build_platform') in SERVO_PLATFORMS
 
     return [l for l, t in graph.tasks.iteritems() if fltr(t)]
