@@ -24,7 +24,12 @@ VERSION_PATH = os.path.join(GECKO, "browser", "config", "version_display.txt")
 
 """Map signing scope aliases to sets of projects.
 
-Currently m-c and m-a use nightly signing; m-b and m-r use release signing.
+Currently m-c and DevEdition on m-b use nightly signing; Beta on m-b and m-r
+use release signing. These data structures aren't set-up to handle different
+scopes on the same repo, so we use a different set of them for DevEdition, and
+callers are responsible for using the correct one (by calling the appropriate
+helper below). More context on this in https://bugzilla.mozilla.org/show_bug.cgi?id=1358601.
+
 We will need to add esr support at some point. Eventually we want to add
 nuance so certain m-b and m-r tasks use dep or nightly signing, and we only
 release sign when we have a signed-off set of candidate builds.  This current
@@ -52,6 +57,17 @@ SIGNING_CERT_SCOPES = {
     'default': 'project:releng:signing:cert:dep-signing',
 }
 
+DEVEDITION_SIGNING_SCOPE_ALIAS_TO_PROJECT = [[
+    'beta', set([
+        'mozilla-beta',
+    ])
+]]
+
+DEVEDITION_SIGNING_CERT_SCOPES = {
+    'beta': 'project:releng:signing:cert:nightly-signing',
+    'default': 'project:releng:signing:cert:dep-signing',
+}
+
 """Map beetmover scope aliases to sets of projects.
 """
 BEETMOVER_SCOPE_ALIAS_TO_PROJECT = [[
@@ -84,6 +100,7 @@ BEETMOVER_SCOPE_ALIAS_TO_TARGET_TASK = [[
     'all-nightly-tasks', set([
         'nightly_fennec',
         'nightly_linux',
+        'nightly_macosx',
         'mozilla_beta_tasks',
         'mozilla_release_tasks',
     ])
@@ -152,7 +169,9 @@ BALROG_SERVER_SCOPES = {
 """
 BALROG_CHANNEL_SCOPES = {
     'nightly': [
-        'project:releng:balrog:channel:nightly'
+        'project:releng:balrog:channel:nightly',
+        'project:releng:balrog:channel:nightly-old-id',
+        'project:releng:balrog:channel:aurora'
     ],
     'aurora': [
         'project:releng:balrog:channel:aurora'
@@ -173,14 +192,15 @@ BALROG_CHANNEL_SCOPES = {
         'project:releng:balrog:channel:esr-cdntest'
     ],
     'default': [
-        'project:releng:balrog:channel:nightly'
+        'project:releng:balrog:channel:nightly',
+        'project:releng:balrog:channel:nightly-old-id',
         'project:releng:balrog:channel:aurora'
         'project:releng:balrog:channel:beta',
         'project:releng:balrog:channel:beta-localtest',
-        'project:releng:balrog:channel:beta-cdntest'
+        'project:releng:balrog:channel:beta-cdntest',
         'project:releng:balrog:channel:release',
         'project:releng:balrog:channel:release-localtest',
-        'project:releng:balrog:channel:release-cdntest'
+        'project:releng:balrog:channel:release-cdntest',
         'project:releng:balrog:channel:esr',
         'project:releng:balrog:channel:esr-localtest',
         'project:releng:balrog:channel:esr-cdntest'
@@ -189,8 +209,8 @@ BALROG_CHANNEL_SCOPES = {
 
 
 PUSH_APK_SCOPE_ALIAS_TO_PROJECT = [[
-    'aurora', set([
-        'mozilla-aurora',
+    'central', set([
+        'mozilla-central',
     ])
 ], [
     'beta', set([
@@ -204,7 +224,7 @@ PUSH_APK_SCOPE_ALIAS_TO_PROJECT = [[
 
 
 PUSH_APK_SCOPES = {
-    'aurora': 'project:releng:googleplay:aurora',
+    'central': 'project:releng:googleplay:aurora',
     'beta': 'project:releng:googleplay:beta',
     'release': 'project:releng:googleplay:release',
     'default': 'project:releng:googleplay:invalid',
@@ -212,21 +232,21 @@ PUSH_APK_SCOPES = {
 
 # See https://github.com/mozilla-releng/pushapkscript#aurora-beta-release-vs-alpha-beta-production
 PUSH_APK_GOOGLE_PLAY_TRACT = {
-    'aurora': 'beta',
+    'central': 'beta',
     'beta': 'production',
     'release': 'rollout',
     'default': 'invalid',
 }
 
 PUSH_APK_BREAKPOINT_WORKER_TYPE = {
-    'aurora': 'aws-provisioner-v1/taskcluster-generic',
+    'central': 'aws-provisioner-v1/taskcluster-generic',
     'beta': 'null-provisioner/human-breakpoint',
     'release': 'null-provisioner/human-breakpoint',
     'default': 'invalid/invalid',
 }
 
 PUSH_APK_DRY_RUN_OPTION = {
-    'aurora': False,
+    'central': False,
     'beta': False,
     'release': False,
     'default': True,
@@ -313,6 +333,12 @@ get_signing_cert_scope = functools.partial(
     SIGNING_CERT_SCOPES
 )
 
+get_devedition_signing_cert_scope = functools.partial(
+    get_scope_from_project,
+    DEVEDITION_SIGNING_SCOPE_ALIAS_TO_PROJECT,
+    DEVEDITION_SIGNING_CERT_SCOPES
+)
+
 get_beetmover_bucket_scope = functools.partial(
     get_scope_from_target_method_and_project,
     BEETMOVER_SCOPE_ALIAS_TO_TARGET_TASK,
