@@ -6,6 +6,7 @@
 
 const {classes: Cc, interfaces: Ci, utils: Cu} = Components;
 
+Cu.import('resource://gre/modules/Services.jsm');
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 Cu.import("resource://gre/modules/Log.jsm");
 
@@ -32,19 +33,6 @@ XPCOMUtils.defineLazyGetter(this, "service", () => {
 
 this.EXPORTED_SYMBOLS = ["accessibility"];
 
-/**
- * Number of attempts to get an accessible object for an element.
- * We attempt more than once because accessible tree can be out of sync
- * with the DOM tree for a short period of time.
- */
-const GET_ACCESSIBLE_ATTEMPTS = 100;
-
-/**
- * An interval between attempts to retrieve an accessible object for an
- * element.
- */
-const GET_ACCESSIBLE_ATTEMPT_INTERVAL = 10;
-
 this.accessibility = {
   get service() {
     return service;
@@ -150,28 +138,47 @@ accessibility.Checks = class {
         return;
       }
 
-      let acc = accessibility.service.getAccessibleFor(element);
-      if (acc || !mustHaveAccessible) {
-        // if accessible object is found, return it;
-        // if it is not required, also resolve
-        resolve(acc);
-      } else {
-        // if we require an accessible object, we need to poll for it
-        // because accessible tree might be
-        // out of sync with DOM tree for a short time
-        let attempts = GET_ACCESSIBLE_ATTEMPTS;
-        let intervalId = setInterval(() => {
-          let acc = accessibility.service.getAccessibleFor(element);
-          if (acc || --attempts <= 0) {
-            clearInterval(intervalId);
-            if (acc) {
-              resolve(acc);
-            } else {
-              reject();
-            }
-          }
-        }, GET_ACCESSIBLE_ATTEMPT_INTERVAL);
+      // First, check if accessibility is ready.
+      let docAcc = accessibility.service.getAccessibleFor(element.ownerDocument);
+      let state = {};
+      docAcc.getState(state, {});
+      if ((state.value & Ci.nsIAccessibleStates.STATE_BUSY) == 0) {
+        // Accessibility is ready, resolve immediately.
+        let acc = accessibility.service.getAccessibleFor(element);
+        if (mustHaveAccessible && !acc) {
+          reject();
+        } else {
+          resolve(acc);
+        }
+        return;
       }
+      // Accessibility for the doc is busy, so wait for the state to change.
+      let eventObserver = {
+        observe(subject, topic, data) {
+          if (topic !== "accessible-event") {
+            return;
+          }
+
+          let event = subject.QueryInterface(Ci.nsIAccessibleEvent);
+          // If event type does not match expected type, skip the event.
+          if (event.eventType !== Ci.nsIAccessibleEvent.EVENT_STATE_CHANGE) {
+            return;
+          }
+          // If event's accessible does not match expected accessible, skip the event.
+          if (event.accessible !== docAcc) {
+            return;
+          }
+
+          Services.obs.removeObserver(this, "accessible-event");
+          let acc = accessibility.service.getAccessibleFor(element);
+          if (mustHaveAccessible && !acc) {
+            reject();
+          } else {
+            resolve(acc);
+          }
+        }
+      };
+      Services.obs.addObserver(eventObserver, "accessible-event");
     }).catch(() => this.error(
         "Element does not have an accessible object", element));
   };
@@ -227,7 +234,7 @@ accessibility.Checks = class {
    *     Accessible object.
    *
    * @return {boolean}
-   *     True if the accesible object has a {@code hidden} attribute,
+   *     True if the accessible object has a {@code hidden} attribute,
    *     false otherwise.
    */
   hasHiddenAttribute(accessible) {
