@@ -4,13 +4,14 @@
 
 "use strict";
 
-const {classes: Cc, interfaces: Ci, utils: Cu} = Components;
+const {classes: Cc, interfaces: Ci, utils: Cu, results: Cr} = Components;
 
 Cu.import("resource://gre/modules/Log.jsm");
 Cu.import("resource://gre/modules/NetUtil.jsm");
 Cu.import("resource://gre/modules/Timer.jsm");
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 
+Cu.import("chrome://marionette/content/element.js");
 Cu.import("chrome://marionette/content/error.js");
 
 const logger = Log.repository.getLogger("Marionette");
@@ -173,6 +174,122 @@ evaluate.sandbox = function (sb, script, args = [], opts = {}) {
   });
 };
 
+/**
+ * Convert any web elements in arbitrary objects to DOM elements by
+ * looking them up in the seen element store.
+ *
+ * @param {?} obj
+ *     Arbitrary object containing web elements.
+ * @param {element.Store} seenEls
+ *     Element store to use for lookup of web element references.
+ * @param {Window} win
+ *     Window.
+ * @param {ShadowRoot} shadowRoot
+ *     Shadow root.
+ *
+ * @return {?}
+ *     Same object as provided by |obj| with the web elements replaced
+ *     by DOM elements.
+ */
+evaluate.fromJSON = function (obj, seenEls, win, shadowRoot = undefined) {
+  switch (typeof obj) {
+    case "boolean":
+    case "number":
+    case "string":
+      return obj;
+
+    case "object":
+      if (obj === null) {
+        return obj;
+      }
+
+      // arrays
+      else if (Array.isArray(obj)) {
+        return obj.map(e => evaluate.fromJSON(e, seenEls, win, shadowRoot));
+      }
+
+      // web elements
+      else if (Object.keys(obj).includes(element.Key) ||
+          Object.keys(obj).includes(element.LegacyKey)) {
+        let uuid = obj[element.Key] || obj[element.LegacyKey];
+        let el = seenEls.get(uuid, {frame: win, shadowRoot: shadowRoot});
+        if (!el) {
+          throw new WebDriverError(`Unknown element: ${uuid}`);
+        }
+        return el;
+      }
+
+      // arbitrary objects
+      else {
+        let rv = {};
+        for (let prop in obj) {
+          rv[prop] = evaluate.fromJSON(obj[prop], seenEls, win, shadowRoot);
+        }
+        return rv;
+      }
+  }
+};
+
+/**
+ * Convert arbitrary objects to JSON-safe primitives that can be
+ * transported over the Marionette protocol.
+ *
+ * Any DOM elements are converted to web elements by looking them up
+ * and/or adding them to the element store provided.
+ *
+ * @param {?} obj
+ *     Object to be marshaled.
+ * @param {element.Store} seenEls
+ *     Element store to use for lookup of web element references.
+ *
+ * @return {?}
+ *     Same object as provided by |obj| with the elements replaced by
+ *     web elements.
+ */
+evaluate.toJSON = function (obj, seenEls) {
+  const t = Object.prototype.toString.call(obj);
+
+  // null
+  if (t == "[object Undefined]" || t == "[object Null]") {
+    return null;
+  }
+
+  // literals
+  else if (t == "[object Boolean]" || t == "[object Number]" || t == "[object String]") {
+    return obj;
+  }
+
+  // Array, NodeList, HTMLCollection, et al.
+  else if (element.isCollection(obj)) {
+    return [...obj].map(el => evaluate.toJSON(el, seenEls));
+  }
+
+  // HTMLElement
+  else if ("nodeType" in obj && obj.nodeType == obj.ELEMENT_NODE) {
+    let uuid = seenEls.add(obj);
+    return element.makeWebElement(uuid);
+  }
+
+  // custom JSON representation
+  else if (typeof obj["toJSON"] == "function") {
+    let unsafeJSON = obj.toJSON();
+    return evaluate.toJSON(unsafeJSON, seenEls);
+  }
+
+  // arbitrary objects + files
+  else {
+    let rv = {};
+    for (let prop in obj) {
+      try {
+        rv[prop] = evaluate.toJSON(obj[prop], seenEls);
+      } catch (e if (e.result == Cr.NS_ERROR_NOT_IMPLEMENTED)) {
+        logger.debug(`Skipping ${prop}: ${e.message}`);
+      }
+    }
+    return rv;
+  }
+};
+
 this.sandbox = {};
 
 /**
@@ -330,166 +447,3 @@ this.Sandboxes = class {
     this.boxes_.clear();
   }
 };
-
-/**
- * Stores scripts imported from the local end through the
- * {@code GeckoDriver#importScript} command.
- *
- * Imported scripts are prepended to the script that is evaluated
- * on each call to {@code GeckoDriver#executeScript},
- * {@code GeckoDriver#executeAsyncScript}, and
- * {@code GeckoDriver#executeJSScript}.
- *
- * Usage:
- *
- *     let importedScripts = new evaluate.ScriptStorage();
- *     importedScripts.add(firstScript);
- *     importedScripts.add(secondScript);
- *
- *     let scriptToEval = importedScripts.concat(script);
- *     // firstScript and secondScript are prepended to script
- *
- */
-evaluate.ScriptStorage = class extends Set {
-
-  /**
-   * Produce a string of all stored scripts.
-   *
-   * The stored scripts are concatenated into a string, with optional
-   * additional scripts then appended.
-   *
-   * @param {...string} addional
-   *     Optional scripts to include.
-   *
-   * @return {string}
-   *     Concatenated string consisting of stored scripts and additional
-   *     scripts, in that order.
-   */
-  concat(...additional) {
-    let rv = "";
-    for (let s of this) {
-      rv = s + rv;
-    }
-    for (let s of additional) {
-      rv = rv + s;
-    }
-    return rv;
-  }
-
-  toJson() {
-    return Array.from(this);
-  }
-
-};
-
-/**
- * Service that enables the script storage service to be queried from
- * content space.
- *
- * The storage can back multiple |ScriptStorage|, each typically belonging
- * to a |Context|.  Since imported scripts' scope are global and not
- * scoped to the current browsing context, all imported scripts are stored
- * in chrome space and fetched by content space as needed.
- *
- * Usage in chrome space:
- *
- *     let service = new evaluate.ScriptStorageService(
- *         [Context.CHROME, Context.CONTENT]);
- *     let storage = service.for(Context.CHROME);
- *     let scriptToEval = storage.concat(script);
- *
- */
-evaluate.ScriptStorageService = class extends Map {
-
-  /**
-   * Create the service.
-   *
-   * An optional array of names for script storages to initially create
-   * can be provided.
-   *
-   * @param {Array.<string>=} initialStorages
-   *     List of names of the script storages to create initially.
-   */
-  constructor(initialStorages = []) {
-    super(initialStorages.map(name => [name, new evaluate.ScriptStorage()]));
-  }
-
-  /**
-   * Retrieve the scripts associated with the given context.
-   *
-   * @param {Context} context
-   *     Context to retrieve the scripts from.
-   *
-   * @return {ScriptStorage}
-   *     Scrips associated with given |context|.
-   */
-  for(context) {
-    return this.get(context);
-  }
-
-  processMessage(msg) {
-    switch (msg.name) {
-      case "Marionette:getImportedScripts":
-        let storage = this.for.apply(this, msg.json);
-        return storage.toJson();
-
-      default:
-        throw new TypeError("Unknown message: " + msg.name);
-    }
-  }
-
-  // TODO(ato): The idea of services in chrome space
-  // can be generalised at some later time (see cookies.js:38).
-  receiveMessage(msg) {
-    try {
-      return this.processMessage(msg);
-    } catch (e) {
-      logger.error(e);
-    }
-  }
-
-};
-
-evaluate.ScriptStorageService.prototype.QueryInterface =
-    XPCOMUtils.generateQI([
-      Ci.nsIMessageListener,
-      Ci.nsISupportsWeakReference,
-    ]);
-
-/**
- * Bridges the script storage in chrome space, to make it possible to
- * retrieve a {@code ScriptStorage} associated with a given
- * {@code Context} from content space.
- *
- * Usage in content space:
- *
- *     let client = new evaluate.ScriptStorageServiceClient(chromeProxy);
- *     let storage = client.for(Context.CONTENT);
- *     let scriptToEval = storage.concat(script);
- *
- */
-evaluate.ScriptStorageServiceClient = class {
-
-  /**
-   * @param {proxy.SyncChromeSender} chromeProxy
-   *     Proxy for communicating with chrome space.
-   */
-  constructor(chromeProxy) {
-    this.chrome = chromeProxy;
-  }
-
-  /**
-   * Retrieve scripts associated with the given context.
-   *
-   * @param {Context} context
-   *     Context to retrieve scripts from.
-   *
-   * @return {ScriptStorage}
-   *     Scripts associated with given |context|.
-   */
-  for(context) {
-    let scripts = this.chrome.getImportedScripts(context)[0];
-    return new evaluate.ScriptStorage(scripts);
-  }
-
-};
