@@ -6,18 +6,11 @@
 // with, instrument, and control Gecko.
 //
 // It is included in Firefox, but not enabled by default unless the
-// --marionette flag is passed or the marionette.enabled preference is
-// set to true.
-
-// Whether or not Marionette is enabled.
-pref("marionette.enabled", false);
+// -marionette flag is passed.
 
 // Port to start Marionette server on.
 pref("marionette.port", 2828);
 
-// Forces client connections to come from a loopback device.
-pref("marionette.forcelocal", true);
-
 // Marionette logging verbosity.  Allowed values are "fatal", "error",
 // "warn", "info", "config", "debug", and "trace".
 pref("marionette.log.level", "info");
