@@ -6,32 +6,45 @@
 
 const {Constructor: CC, classes: Cc, interfaces: Ci, utils: Cu} = Components;
 
-const loader = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);
-const ServerSocket = CC("@mozilla.org/network/server-socket;1", "nsIServerSocket", "initSpecialConnection");
+const loader = Cc["@mozilla.org/moz/jssubscript-loader;1"]
+    .getService(Ci.mozIJSSubScriptLoader);
+const ServerSocket = CC(
+    "@mozilla.org/network/server-socket;1",
+    "nsIServerSocket",
+    "initSpecialConnection");
 
 Cu.import("resource://gre/modules/Log.jsm");
 Cu.import("resource://gre/modules/Preferences.jsm");
 Cu.import("resource://gre/modules/Services.jsm");
 Cu.import("resource://gre/modules/Task.jsm");
+Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 
 Cu.import("chrome://marionette/content/assert.js");
 Cu.import("chrome://marionette/content/driver.js");
 Cu.import("chrome://marionette/content/error.js");
 Cu.import("chrome://marionette/content/message.js");
+Cu.import("chrome://marionette/content/transport.js");
 
-// Bug 1083711: Load transport.js as an SDK module instead of subscript
-loader.loadSubScript("resource://devtools/shared/transport/transport.js");
+XPCOMUtils.defineLazyServiceGetter(
+    this, "env", "@mozilla.org/process/environment;1", "nsIEnvironment");
 
 const logger = Log.repository.getLogger("Marionette");
 
+const {KeepWhenOffline, LoopbackOnly} = Ci.nsIServerSocket;
+
 this.EXPORTED_SYMBOLS = ["server"];
 this.server = {};
 
 const PROTOCOL_VERSION = 3;
 
+const ENV_ENABLED = "MOZ_MARIONETTE";
+
 const PREF_CONTENT_LISTENER = "marionette.contentListener";
+const PREF_PORT = "marionette.port";
 const PREF_RECOMMENDED = "marionette.prefs.recommended";
 
+const NOTIFY_RUNNING = "remote-active";
+
 // Marionette sets preferences recommended for automation when it starts,
 // unless |marionette.prefs.recommended| has been set to false.
 // Where noted, some prefs should also be set in the profile passed to
@@ -92,17 +105,12 @@ const RECOMMENDED_PREFS = new Map([
   // thumbnails in general cannot hurt
   ["browser.pagethumbnails.capturing_disabled", true],
 
-  // Avoid performing Reader Mode intros during tests
-  ["browser.reader.detectedFirstArticle", true],
-
   // Disable safebrowsing components.
   //
   // These should also be set in the profile prior to starting Firefox,
   // as it is picked up at runtime.
   ["browser.safebrowsing.blockedURIs.enabled", false],
   ["browser.safebrowsing.downloads.enabled", false],
-  ["browser.safebrowsing.enabled", false],
-  ["browser.safebrowsing.forbiddenURIs.enabled", false],
   ["browser.safebrowsing.malware.enabled", false],
   ["browser.safebrowsing.phishing.enabled", false],
 
@@ -126,8 +134,8 @@ const RECOMMENDED_PREFS = new Map([
   // Do not redirect user when a milstone upgrade of Firefox is detected
   ["browser.startup.homepage_override.mstone", "ignore"],
 
-  // Disable tab animation
-  ["browser.tabs.animate", false],
+  // Disable browser animations
+  ["toolkit.cosmeticAnimations.enabled", false],
 
   // Do not allow background tabs to be zombified, otherwise for tests
   // that open additional tabs, the test harness tab itself might get
@@ -214,9 +222,6 @@ const RECOMMENDED_PREFS = new Map([
   // Show chrome errors and warnings in the error console
   ["javascript.options.showInConsole", true],
 
-  // Make sure the disk cache doesn't get auto disabled
-  ["network.http.bypass-cachelock-threshold", 200000],
-
   // Do not prompt for temporary redirects
   ["network.http.prompt-temp-redirect", false],
 
@@ -268,13 +273,10 @@ server.TCPListener = class {
   /**
    * @param {number} port
    *     Port for server to listen to.
-   * @param {boolean=} forceLocal
-   *     Listen only to connections from loopback if true (default).
-   *     When false, accept all connections.
    */
-  constructor (port, forceLocal = true) {
+  constructor (port) {
     this.port = port;
-    this.forceLocal = forceLocal;
+    this.socket = null;
     this.conns = new Set();
     this.nextConnID = 0;
     this.alive = false;
@@ -285,7 +287,7 @@ server.TCPListener = class {
   /**
    * Function produces a GeckoDriver.
    *
-   * Determines application nameto initialise the driver with.
+   * Determines application name to initialise the driver with.
    *
    * @return {GeckoDriver}
    *     A driver instance.
@@ -305,11 +307,20 @@ server.TCPListener = class {
     this._acceptConnections = value;
   }
 
+  /**
+   * Bind this listener to |port| and start accepting incoming socket
+   * connections on |onSocketAccepted|.
+   *
+   * The marionette.port preference will be populated with the value
+   * of |this.port|.
+   */
   start () {
     if (this.alive) {
       return;
     }
 
+    Services.obs.notifyObservers(this, NOTIFY_RUNNING, true);
+
     if (Preferences.get(PREF_RECOMMENDED)) {
       // set recommended prefs if they are not already user-defined
       for (let [k, v] of RECOMMENDED_PREFS) {
@@ -321,17 +332,16 @@ server.TCPListener = class {
       }
     }
 
-    let flags = Ci.nsIServerSocket.KeepWhenOffline;
-    if (this.forceLocal) {
-      flags |= Ci.nsIServerSocket.LoopbackOnly;
-    } else {
-      logger.warn("Server socket is not limited to loopback connections");
-    }
-    this.listener = new ServerSocket(this.port, flags, 1);
-    this.listener.asyncListen(this);
+    const flags = KeepWhenOffline | LoopbackOnly;
+    const backlog = 1;
+    this.socket = new ServerSocket(this.port, flags, backlog);
+    this.socket.asyncListen(this);
+    this.port = this.socket.port;
+    Preferences.set(PREF_PORT, this.port);
 
     this.alive = true;
     this._acceptConnections = true;
+    env.set(ENV_ENABLED, "1");
   }
 
   stop () {
@@ -339,20 +349,20 @@ server.TCPListener = class {
       return;
     }
 
+    this._acceptConnections = false;
+
+    this.socket.close();
+    this.socket = null;
+
     for (let k of this.alteredPrefs) {
       logger.debug(`Resetting recommended pref ${k}`);
       Preferences.reset(k);
     }
-    this.closeListener();
-
     this.alteredPrefs.clear();
-    this.alive = false;
-    this._acceptConnections = false;
-  }
 
-  closeListener () {
-    this.listener.close();
-    this.listener = null;
+    Services.obs.notifyObservers(this, NOTIFY_RUNNING, false);
+
+    this.alive = false;
   }
 
   onSocketAccepted (serverSocket, clientSocket) {
