@@ -3,6 +3,8 @@
  * Generally it just delegates to EventUtils.js.
  */
 
+Components.utils.import("resource://gre/modules/Services.jsm");
+
 // Set up a dummy environment so that EventUtils works. We need to be careful to
 // pass a window object into each EventUtils method we call rather than having
 // it rely on the |window| global.
@@ -37,6 +39,27 @@ addMessageListener("Test:SynthesizeMouse", (message) => {
   let left = data.x;
   let top = data.y;
   if (target) {
+    if (target.ownerDocument !== content.document) {
+      // Account for nodes found in iframes.
+      let cur = target;
+      do {
+        let frame = cur.ownerGlobal.frameElement;
+        let rect = frame.getBoundingClientRect();
+
+        left += rect.left;
+        top += rect.top;
+
+        cur = frame;
+      } while (cur && cur.ownerDocument !== content.document);
+
+      // node must be in this document tree.
+      if (!cur) {
+        sendAsyncMessage("Test:SynthesizeMouseDone",
+                         { error: "target must be in the main document tree" });
+        return;
+      }
+    }
+
     let rect = target.getBoundingClientRect();
     left += rect.left;
     top += rect.top;
