@@ -3,8 +3,9 @@ config = {
     'stage_platform': 'android-checkstyle',
     'build_type': 'api-15-opt',
     'src_mozconfig': 'mobile/android/config/mozconfigs/android-api-15-frontend/nightly',
-    'tooltool_manifest_src': 'mobile/android/config/tooltool-manifests/android-frontend/releng.manifest',
     'multi_locale_config_platform': 'android',
+    # checkstyle doesn't produce a package. So don't collect package metrics.
+    'disable_package_metrics': True,
     'postflight_build_mach_commands': [
         ['gradle', 'app:checkstyle'],
     ],
