@@ -7,8 +7,8 @@
 # ***** END LICENSE BLOCK *****
 """ gecko_migration.py
 
-Merge day script for gecko (mozilla-central -> mozilla-aurora,
-mozilla-aurora -> mozilla-beta, mozilla-beta -> mozilla-release).
+Merge day script for gecko (mozilla-central -> mozilla-beta,
+mozilla-beta -> mozilla-release).
 
 Ported largely from
 http://hg.mozilla.org/build/tools/file/084bc4e2fc76/release/beta2release.py
@@ -16,29 +16,31 @@ and
 http://hg.mozilla.org/build/tools/file/084bc4e2fc76/release/merge_helper.py
 """
 
-from getpass import getpass
 import os
 import pprint
 import subprocess
 import sys
+from getpass import getpass
 
 sys.path.insert(1, os.path.dirname(os.path.dirname(sys.path[0])))
 
 from mozharness.base.errors import HgErrorList
-from mozharness.base.log import INFO, FATAL
 from mozharness.base.python import VirtualenvMixin, virtualenv_config_options
 from mozharness.base.vcs.vcsbase import MercurialScript
-from mozharness.base.vcs.mercurial import MercurialVCS
 from mozharness.mozilla.selfserve import SelfServeMixin
 from mozharness.mozilla.updates.balrog import BalrogMixin
+from mozharness.mozilla.buildbot import BuildbotMixin
+from mozharness.mozilla.repo_manipulation import MercurialRepoManipulationMixin
 
 VALID_MIGRATION_BEHAVIORS = (
-    "beta_to_release", "aurora_to_beta", "central_to_aurora", "release_to_esr"
+    "beta_to_release", "central_to_beta", "release_to_esr", "bump_second_digit",
 )
 
 
 # GeckoMigration {{{1
-class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMixin):
+class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin,
+                     SelfServeMixin, BuildbotMixin,
+                     MercurialRepoManipulationMixin):
     config_options = [
         [['--hg-user', ], {
             "action": "store",
@@ -84,6 +86,7 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
                 'pull',
                 'lock-update-paths',
                 'migrate',
+                'bump_second_digit',
                 'commit-changes',
                 'push',
                 'trigger-builders',
@@ -126,16 +129,16 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
             dirs['abs_work_dir'], 'tools', 'lib', 'python'
         )
         for k in ('from', 'to'):
-            dir_name = self.config.get(
-                "%s_repo_dir",
-                self.get_filename_from_url(self.config["%s_repo_url" % k])
-            )
-            self.abs_dirs['abs_%s_dir' % k] = os.path.join(
-                dirs['abs_work_dir'], dir_name
-            )
+            url = self.config.get("%s_repo_url" % k)
+            if url:
+                dir_name = self.get_filename_from_url(url)
+                self.info("adding %s" % dir_name)
+                self.abs_dirs['abs_%s_dir' % k] = os.path.join(
+                    dirs['abs_work_dir'], dir_name
+                )
         return self.abs_dirs
 
-    def query_gecko_repos(self):
+    def query_repos(self):
         """ Build a list of repos to clone.
             """
         if self.gecko_repos:
@@ -144,23 +147,45 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
         dirs = self.query_abs_dirs()
         self.gecko_repos = []
         for k in ('from', 'to'):
-            url = self.config["%s_repo_url" % k]
-            self.gecko_repos.append({
-                "repo": url,
-                "revision": self.config.get("%s_repo_revision", "default"),
-                "dest": dirs['abs_%s_dir' % k],
-                "vcs": "hg",
-            })
+            repo_key = "%s_repo_url" % k
+            url = self.config.get(repo_key)
+            if url:
+                self.gecko_repos.append({
+                    "repo": url,
+                    "branch": self.config.get("%s_repo_branch" % (k,), "default"),
+                    "dest": dirs['abs_%s_dir' % k],
+                    "vcs": "hg",
+                    # "hg" vcs uses robustcheckout extension requires the use of a share
+                    # but having a share breaks migration logic when merging repos.
+                    # Solution: tell hg vcs to create a unique share directory for each
+                    # gecko repo. see mozharness/base/vcs/mercurial.py for implementation
+                    "use_vcs_unique_share": True,
+                })
+            else:
+                self.warning("Skipping %s" % repo_key)
         self.info(pprint.pformat(self.gecko_repos))
         return self.gecko_repos
 
-    def query_hg_revision(self, path):
-        """ Avoid making 'pull' a required action every run, by being able
-            to fall back to figuring out the revision from the cloned repo
-            """
-        m = MercurialVCS(log_obj=self.log_obj, config=self.config)
-        revision = m.get_revision_from_path(path)
-        return revision
+    def query_commit_dirs(self):
+        dirs = self.query_abs_dirs()
+        commit_dirs = [dirs['abs_to_dir']]
+        if self.config['migration_behavior'] == 'central_to_beta':
+            commit_dirs.append(dirs['abs_from_dir'])
+        return commit_dirs
+
+    def query_commit_message(self):
+        return "Update configs. IGNORE BROKEN CHANGESETS CLOSED TREE NO BUG a=release ba=release"
+
+    def query_push_dirs(self):
+        dirs = self.query_abs_dirs()
+        return dirs.get('abs_from_dir'), dirs.get('abs_to_dir')
+
+    def query_push_args(self, cwd):
+        if cwd == self.query_abs_dirs()['abs_to_dir'] and \
+                self.config['migration_behavior'] == 'beta_to_release':
+            return ['--new-branch', '-r', '.']
+        else:
+            return ['-r', '.']
 
     def query_from_revision(self):
         """ Shortcut to get the revision for the from repo
@@ -174,53 +199,6 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
         dirs = self.query_abs_dirs()
         return self.query_hg_revision(dirs['abs_to_dir'])
 
-    def get_fx_major_version(self, path):
-        version_path = os.path.join(path, "browser", "config", "version.txt")
-        contents = self.read_from_file(version_path, error_level=FATAL)
-        return contents.split(".")[0]
-
-    def hg_tag(self, cwd, tags, user=None, message=None, revision=None,
-               force=None, halt_on_failure=True):
-        if isinstance(tags, basestring):
-            tags = [tags]
-        message = "No bug - Tagging %s" % os.path.basename(cwd)
-        if revision:
-            message = "%s %s" % (message, revision)
-        message = "%s with %s" % (message, ', '.join(tags))
-        message += " a=release DONTBUILD CLOSED TREE"
-        self.info(message)
-        cmd = self.query_exe('hg', return_type='list') + ['tag']
-        if user:
-            cmd.extend(['-u', user])
-        if message:
-            cmd.extend(['-m', message])
-        if revision:
-            cmd.extend(['-r', revision])
-        if force:
-            cmd.append('-f')
-        cmd.extend(tags)
-        return self.run_command(
-            cmd, cwd=cwd, halt_on_failure=halt_on_failure,
-            error_list=HgErrorList
-        )
-
-    def hg_commit(self, cwd, message, user=None, ignore_no_changes=False):
-        """ Commit changes to hg.
-            """
-        cmd = self.query_exe('hg', return_type='list') + [
-            'commit', '-m', message]
-        if user:
-            cmd.extend(['-u', user])
-        success_codes = [0]
-        if ignore_no_changes:
-            success_codes.append(1)
-        self.run_command(
-            cmd, cwd=cwd, error_list=HgErrorList,
-            halt_on_failure=True,
-            success_codes=success_codes
-        )
-        return self.query_hg_revision(cwd)
-
     def hg_merge_via_debugsetparents(self, cwd, old_head, new_head,
                                      preserve_tags=True, user=None):
         """ Merge 2 heads avoiding non-fastforward commits
@@ -284,16 +262,6 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
             else:
                 self.info(".hgtags file is identical, no need to commit")
 
-    def replace(self, file_name, from_, to_):
-        """ Replace text in a file.
-            """
-        text = self.read_from_file(file_name, error_level=FATAL)
-        new_text = text.replace(from_, to_)
-        if text == new_text:
-            self.fatal("Cannot replace '%s' to '%s' in '%s'" %
-                       (from_, to_, file_name))
-        self.write_to_file(file_name, new_text, error_level=FATAL)
-
     def remove_locales(self, file_name, locales):
         """ Remove locales from shipped-locales (m-r only)
             """
@@ -319,16 +287,20 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
         self.write_to_file(clobber_file, new_contents)
 
     def bump_version(self, cwd, curr_version, next_version, curr_suffix,
-                     next_suffix, bump_major=False):
-        """ Bump versions (m-c, m-a, m-b).
+                     next_suffix, bump_major=False, use_config_suffix=False):
+        """ Bump versions (m-c, m-b).
 
             At some point we may want to unhardcode these filenames into config
             """
         curr_weave_version = str(int(curr_version) + 2)
         next_weave_version = str(int(curr_weave_version) + 1)
         for f in self.config["version_files"]:
-            self.replace(os.path.join(cwd, f), "%s.0%s" % (curr_version, curr_suffix),
-                         "%s.0%s" % (next_version, next_suffix))
+            from_ = "%s.0%s" % (curr_version, curr_suffix)
+            if use_config_suffix:
+                to = "%s.0%s%s" % (next_version, next_suffix, f["suffix"])
+            else:
+                to = "%s.0%s" % (next_version, next_suffix)
+            self.replace(os.path.join(cwd, f["file"]), from_, to)
 
         # only applicable for m-c
         if bump_major:
@@ -344,8 +316,8 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
             )
 
     # Branch-specific workflow helper methods {{{1
-    def central_to_aurora(self, end_tag):
-        """ mozilla-central -> mozilla-aurora behavior.
+    def central_to_beta(self, end_tag):
+        """ mozilla-central -> mozilla-beta behavior.
 
             We could have all of these individually toggled by flags, but
             by separating into workflow methods we can be more precise about
@@ -353,48 +325,21 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
             staging beta user repo migrations.
             """
         dirs = self.query_abs_dirs()
-        self.info("Reverting locales")
-        hg = self.query_exe("hg", return_type="list")
-        for f in self.config["locale_files"]:
-            self.run_command(
-                hg + ["revert", "-r", end_tag, f],
-                cwd=dirs['abs_to_dir'],
-                error_list=HgErrorList,
-                halt_on_failure=True,
-            )
-        next_ma_version = self.get_fx_major_version(dirs['abs_to_dir'])
-        self.bump_version(dirs['abs_to_dir'], next_ma_version, next_ma_version, "a1", "a2")
+        next_mb_version = self.get_version(dirs['abs_to_dir'])[0]
+        self.bump_version(dirs['abs_to_dir'], next_mb_version, next_mb_version, "a1", "", use_config_suffix=True)
         self.apply_replacements()
         # bump m-c version
-        curr_mc_version = self.get_fx_major_version(dirs['abs_from_dir'])
+        curr_mc_version = self.get_version(dirs['abs_from_dir'])[0]
         next_mc_version = str(int(curr_mc_version) + 1)
         self.bump_version(
             dirs['abs_from_dir'], curr_mc_version, next_mc_version, "a1", "a1",
-            bump_major=True
+            bump_major=True,
+            use_config_suffix=False
         )
         # touch clobber files
         self.touch_clobber_file(dirs['abs_from_dir'])
         self.touch_clobber_file(dirs['abs_to_dir'])
 
-    def aurora_to_beta(self, *args, **kwargs):
-        """ mozilla-aurora -> mozilla-beta behavior.
-
-            We could have all of these individually toggled by flags, but
-            by separating into workflow methods we can be more precise about
-            what happens in each workflow, while allowing for things like
-            staging beta user repo migrations.
-            """
-        dirs = self.query_abs_dirs()
-        mb_version = self.get_fx_major_version(dirs['abs_to_dir'])
-        self.bump_version(dirs['abs_to_dir'], mb_version, mb_version, "a2", "")
-        self.apply_replacements()
-        self.touch_clobber_file(dirs['abs_to_dir'])
-        # TODO mozconfig diffing
-        # The build/tools version only checks the mozconfigs from hgweb, so
-        # can't help pre-push.  The in-tree mozconfig diffing requires a mach
-        # virtualenv to be installed.  If we want this sooner we can put this
-        # in the push action; otherwise we may just wait until we have in-tree
-        # mozconfig checking.
 
     def beta_to_release(self, *args, **kwargs):
         """ mozilla-beta -> mozilla-release behavior.
@@ -405,6 +350,12 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
             staging beta user repo migrations.
             """
         dirs = self.query_abs_dirs()
+        # Reset display_version.txt
+        for f in self.config["copy_files"]:
+            self.copyfile(
+                os.path.join(dirs['abs_to_dir'], f["src"]),
+                os.path.join(dirs['abs_to_dir'], f["dst"]))
+
         self.apply_replacements()
         if self.config.get("remove_locales"):
             self.remove_locales(
@@ -416,10 +367,9 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
     def release_to_esr(self, *args, **kwargs):
         """ mozilla-release -> mozilla-esrNN behavior. """
         dirs = self.query_abs_dirs()
-        for to_transplant in self.config.get("transplant_patches", []):
-            self.transplant(repo=to_transplant["repo"],
-                            changeset=to_transplant["changeset"],
-                            cwd=dirs['abs_to_dir'])
+        for to_graft in self.config.get("graft_patches", []):
+            self.graft(repo=to_graft["repo"], changeset=to_graft["changeset"],
+                       cwd=dirs['abs_to_dir'])
         self.apply_replacements()
         self.touch_clobber_file(dirs['abs_to_dir'])
 
@@ -428,20 +378,27 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
         for f, from_, to in self.config["replacements"]:
             self.replace(os.path.join(dirs['abs_to_dir'], f), from_, to)
 
-    def transplant(self, repo, changeset, cwd):
-        """Transplant a Mercurial changeset from a remote repository."""
+    def graft(self, repo, changeset, cwd):
+        """Graft a Mercurial changeset from a remote repository."""
         hg = self.query_exe("hg", return_type="list")
-        cmd = hg + ["--config", "extensions.transplant=", "transplant",
-                    "--source", repo, changeset]
-        self.info("Transplanting %s from %s" % (changeset, repo))
+        self.info("Pulling %s from %s" % (changeset, repo))
+        pull_cmd = hg + ["pull", "-r", changeset, repo]
+        status = self.run_command(
+            pull_cmd,
+            cwd=cwd,
+            error_list=HgErrorList,
+        )
+        if status != 0:
+            self.fatal("Cannot pull %s from %s properly" % (changeset, repo))
+        cmd = hg + ["graft", changeset]
+        self.info("Grafting %s from %s" % (changeset, repo))
         status = self.run_command(
             cmd,
             cwd=cwd,
             error_list=HgErrorList,
         )
         if status != 0:
-            self.fatal("Cannot transplant %s from %s properly" %
-                       (changeset, repo))
+            self.fatal("Cannot graft %s from %s properly" % (changeset, repo))
 
     def pull_from_repo(self, from_dir, to_dir, revision=None, branch=None):
         """ Pull from one repo to another. """
@@ -467,64 +424,33 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
         )
 
 # Actions {{{1
-    def clean_repos(self):
-        """ We may end up with contaminated local repos at some point, but
-            we don't want to have to clobber and reclone from scratch every
-            time.
+    def bump_second_digit(self, *args, **kwargs):
+        """Bump second digit.
 
-            This is an attempt to clean up the local repos without needing a
-            clobber.
-            """
+         ESR need only the second digit bumped as a part of merge day."""
         dirs = self.query_abs_dirs()
-        hg = self.query_exe("hg", return_type="list")
-        hg_repos = self.query_gecko_repos()
-        hg_strip_error_list = [{
-            'substr': r'''abort: empty revision set''', 'level': INFO,
-            'explanation': "Nothing to clean up; we're good!",
-        }] + HgErrorList
-        for repo_config in hg_repos:
-            repo_name = repo_config["dest"]
-            repo_path = os.path.join(dirs['abs_work_dir'], repo_name)
-            if os.path.exists(repo_path):
-                # hg up -C to discard uncommitted changes
-                self.run_command(
-                    hg + ["up", "-C", "-r", repo_config['revision']],
-                    cwd=repo_path,
-                    error_list=HgErrorList,
-                    halt_on_failure=True,
-                )
-                # discard unpushed commits
-                status = self.retry(
-                    self.run_command,
-                    args=(hg + ["--config", "extensions.mq=", "strip",
-                          "--no-backup", "outgoing()"], ),
-                    kwargs={
-                        'cwd': repo_path,
-                        'error_list': hg_strip_error_list,
-                        'return_type': 'num_errors',
-                        'success_codes': (0, 255),
-                    },
-                )
-                if status not in [0, 255]:
-                    self.fatal("Issues stripping outgoing revisions!")
-                # 2nd hg up -C to make sure we're not on a stranded head
-                # which can happen when reverting debugsetparents
-                self.run_command(
-                    hg + ["up", "-C", "-r", repo_config['revision']],
-                    cwd=repo_path,
-                    error_list=HgErrorList,
-                    halt_on_failure=True,
-                )
+        version = self.get_version(dirs['abs_to_dir'])
+        curr_version = ".".join(version)
+        next_version = list(version)
+        # bump the second digit
+        next_version[1] = str(int(next_version[1]) + 1)
+        # Take major+minor and append '0' accordng to Firefox version schema.
+        # 52.0 will become 52.1.0, not 52.1
+        next_version = ".".join(next_version[:2] + ['0'])
+        for f in self.config["version_files"]:
+            self.replace(os.path.join(dirs['abs_to_dir'], f["file"]),
+                         curr_version, next_version + f["suffix"])
+        self.touch_clobber_file(dirs['abs_to_dir'])
 
     def pull(self):
-        """ Pull tools first, then use hgtool for the gecko repos
+        """ Pull tools first, then clone the gecko repos
             """
         repos = [{
             "repo": self.config["tools_repo_url"],
-            "revision": self.config["tools_repo_revision"],
+            "branch": self.config["tools_repo_branch"],
             "dest": "tools",
             "vcs": "hg",
-        }] + self.query_gecko_repos()
+        }] + self.query_repos()
         super(GeckoMigration, self).pull(repos=repos)
 
     def lock_update_paths(self):
@@ -534,16 +460,13 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
         """ Perform the migration.
             """
         dirs = self.query_abs_dirs()
-        from_fx_major_version = self.get_fx_major_version(dirs['abs_from_dir'])
-        to_fx_major_version = self.get_fx_major_version(dirs['abs_to_dir'])
+        from_fx_major_version = self.get_version(dirs['abs_from_dir'])[0]
+        to_fx_major_version = self.get_version(dirs['abs_to_dir'])[0]
         base_from_rev = self.query_from_revision()
         base_to_rev = self.query_to_revision()
         base_tag = self.config['base_tag'] % {'major_version': from_fx_major_version}
-        end_tag = self.config['end_tag'] % {'major_version': to_fx_major_version}
         self.hg_tag(
             dirs['abs_from_dir'], base_tag, user=self.config['hg_user'],
-            message="Added %s tag for changeset %s. IGNORE BROKEN CHANGESETS DONTBUILD CLOSED TREE NO BUG a=release" %
-                    (base_tag, base_from_rev),
             revision=base_from_rev,
         )
         new_from_rev = self.query_from_revision()
@@ -561,63 +484,20 @@ class GeckoMigration(MercurialScript, BalrogMixin, VirtualenvMixin, SelfServeMix
                 dirs['abs_to_dir'], old_head=base_to_rev, new_head=new_from_rev,
                 user=self.config['hg_user'],
             )
-        self.hg_tag(
-            dirs['abs_to_dir'], end_tag, user=self.config['hg_user'],
-            message="Added %s tag for changeset %s. IGNORE BROKEN CHANGESETS DONTBUILD CLOSED TREE NO BUG a=release" %
-                    (end_tag, base_to_rev),
-            revision=base_to_rev, force=True,
-        )
+
+        end_tag = self.config.get('end_tag')
+        if end_tag:
+            end_tag = end_tag % {'major_version': to_fx_major_version}
+            self.hg_tag(
+                dirs['abs_to_dir'], end_tag, user=self.config['hg_user'],
+                revision=base_to_rev, force=True,
+            )
         # Call beta_to_release etc.
         if not hasattr(self, self.config['migration_behavior']):
             self.fatal("Don't know how to proceed with migration_behavior %s !" % self.config['migration_behavior'])
         getattr(self, self.config['migration_behavior'])(end_tag=end_tag)
         self.info("Verify the diff, and apply any manual changes, such as disabling features, and --commit-changes")
 
-    def commit_changes(self):
-        """ Do the commit.
-            """
-        hg = self.query_exe("hg", return_type="list")
-        dirs = self.query_abs_dirs()
-        commit_dirs = [dirs['abs_to_dir']]
-        if self.config['migration_behavior'] == 'central_to_aurora':
-            commit_dirs.append(dirs['abs_from_dir'])
-        for cwd in commit_dirs:
-            self.run_command(hg + ["diff"], cwd=cwd)
-            self.hg_commit(
-                cwd, user=self.config['hg_user'],
-                message="Update configs. IGNORE BROKEN CHANGESETS CLOSED TREE NO BUG a=release ba=release"
-            )
-        self.info("Now verify |hg out| and |hg out --patch| if you're paranoid, and --push")
-
-    def push(self):
-        """
-            """
-        error_message = """Push failed!  If there was a push race, try rerunning
-the script (--clean-repos --pull --migrate).  The second run will be faster."""
-        dirs = self.query_abs_dirs()
-        hg = self.query_exe("hg", return_type="list")
-        for cwd in (dirs['abs_from_dir'], dirs['abs_to_dir']):
-            push_cmd = hg + ['push']
-            if cwd == dirs['abs_to_dir'] and self.config['migration_behavior'] == 'beta_to_release':
-                push_cmd.append('--new-branch')
-            status = self.run_command(
-                push_cmd,
-                cwd=cwd,
-                error_list=HgErrorList,
-                success_codes=[0, 1],
-            )
-            if status == 1:
-                self.warning("No changes for %s!" % cwd)
-            elif status:
-                if cwd == dirs['abs_from_dir'] and self.config['migration_behavior'] == 'central_to_aurora':
-                    message = """m-c push failed!
-You may be able to fix by |hg rebase| and rerunning --push if successful.
-If not, try rerunning the script (--clean-repos --pull --migrate).
-The second run will be faster."""
-                else:
-                    message = error_message
-                self.fatal(message)
-
     def trigger_builders(self):
         """Triggers builders that should be run directly after a merge.
         There are two different types of things we trigger:
@@ -628,7 +508,7 @@ The second run will be faster."""
            very quickly).
         2) Arbitrary builders ("post_merge_builders"). These are additional
            builders to trigger that aren't part of the nightly builder set.
-           For example: hg bundle generation builders.
+           Previous example: hg bundle generation builders.
         """
         dirs = self.query_abs_dirs()
         branch = self.config["to_repo_url"].rstrip("/").split("/")[-1]
@@ -646,5 +526,4 @@ The second run will be faster."""
 
 # __main__ {{{1
 if __name__ == '__main__':
-    gecko_migration = GeckoMigration()
-    gecko_migration.run_and_exit()
+    GeckoMigration().run_and_exit()
