@@ -22,17 +22,17 @@ class _StopAction(argparse.Action):
 class _ListTests(_StopAction):
     def __call__(self, parser, namespace, values, option_string=None):
         from talos import test
-        print 'Available tests:'
-        print '================\n'
+        print('Available tests:')
+        print('================\n')
         test_class_names = [
             (test_class.name(), test_class.description())
             for test_class in test.test_dict().itervalues()
         ]
         test_class_names.sort()
         for name, description in test_class_names:
-            print name
-            print '-'*len(name)
-            print description
+            print(name)
+            print('-'*len(name))
+            print(description)
             print  # Appends a single blank line to the end
         parser.exit()
 
@@ -40,12 +40,12 @@ class _ListTests(_StopAction):
 class _ListSuite(_StopAction):
     def __call__(self, parser, namespace, values, option_string=None):
         from talos.config import suites_conf
-        print 'Available suites:'
+        print('Available suites:')
         conf = suites_conf()
         max_suite_name = max([len(s) for s in conf])
         pattern = " %%-%ds (%%s)" % max_suite_name
         for name in conf:
-            print pattern % (name, ':'.join(conf[name]['tests']))
+            print(pattern % (name, ':'.join(conf[name]['tests'])))
         print
         parser.exit()
 
@@ -67,8 +67,9 @@ def create_parser(mach_interface=False):
             help="List of tests to run, separated by ':' (ex. damp:cart)")
     add_arg('--suite',
             help="Suite to use (instead of --activeTests)")
-    add_arg('--e10s', action='store_true',
-            help="enable e10s")
+    add_arg('--disable-e10s', dest='e10s',
+            action='store_false', default=True,
+            help="disable e10s")
     add_arg('--noChrome', action='store_true',
             help="do not run tests as chrome")
     add_arg('--rss', action='store_true',
@@ -79,11 +80,20 @@ def create_parser(mach_interface=False):
                  " an environment variable")
     add_arg("--mozAfterPaint", action='store_true', dest="tpmozafterpaint",
             help="wait for MozAfterPaint event before recording the time")
-    add_arg('--spsProfile', action="store_true", dest="sps_profile",
-            help="Profile the run and output the results in $MOZ_UPLOAD_DIR")
-    add_arg('--spsProfileInterval', dest='sps_profile_interval', type=int,
+    add_arg('--spsProfile', action="store_true", dest="gecko_profile",
+            help="(Deprecated - Use --geckoProfile instead.) Profile the "
+                 "run and output the results in $MOZ_UPLOAD_DIR.")
+    add_arg('--spsProfileInterval', dest='gecko_profile_interval', type=float,
+            help="(Deprecated - Use --geckoProfileInterval instead.) How "
+                 "frequently to take samples (ms)")
+    add_arg('--spsProfileEntries', dest="gecko_profile_entries", type=int,
+            help="(Deprecated - Use --geckoProfileEntries instead.) How "
+                 "many samples to take with the profiler")
+    add_arg('--geckoProfile', action="store_true", dest="gecko_profile",
+            help="Profile the run and output the results in $MOZ_UPLOAD_DIR.")
+    add_arg('--geckoProfileInterval', dest='gecko_profile_interval', type=float,
             help="How frequently to take samples (ms)")
-    add_arg('--spsProfileEntries', dest="sps_profile_entries", type=int,
+    add_arg('--geckoProfileEntries', dest="gecko_profile_entries", type=int,
             help="How many samples to take with the profiler")
     add_arg('--extension', dest='extensions', action='append',
             default=['${talos}/talos-powers/talos-powers-signed.xpi',
@@ -103,17 +113,22 @@ def create_parser(mach_interface=False):
                  " Currently used for xperf only.")
     add_arg('--noShutdown', dest='shutdown', action='store_true',
             help="Record time browser takes to shutdown after testing")
-    add_arg('--setPref', action='append', default=[], dest="extraPrefs",
+    add_arg('--setpref', action='append', default=[], dest="extraPrefs",
             metavar="PREF=VALUE",
             help="defines an extra user preference")
+    add_arg('--mitmproxy',
+            help='Test uses mitmproxy to serve the pages, specify the '
+                 'path and name of the mitmdump file to playback')
+    add_arg('--mitmdumpPath',
+            help="Path to mitmproxy's mitmdump playback tool")
+    add_arg("--firstNonBlankPaint", action='store_true', dest="first_non_blank_paint",
+            help="Wait for firstNonBlankPaint event before recording the time")
     add_arg('--webServer', dest='webserver',
             help="DEPRECATED")
     if not mach_interface:
         add_arg('--develop', action='store_true', default=False,
                 help="useful for running tests on a developer machine."
                      " Doesn't upload to the graph servers.")
-    add_arg('--responsiveness', action='store_true',
-            help="turn on responsiveness collection")
     add_arg("--cycles", type=int,
             help="number of browser cycles to run")
     add_arg("--tpmanifest",
@@ -128,6 +143,8 @@ def create_parser(mach_interface=False):
                  ' the manifest')
     add_arg('--tpdelay', type=int,
             help="length of the pageloader delay")
+    add_arg('--no-download', action="store_true", dest="no_download",
+            help="Do not download the talos test pagesets")
     add_arg('--sourcestamp',
             help='Specify the hg revision or sourcestamp for the changeset'
                  ' we are testing.  This will use the value found in'
@@ -136,6 +153,10 @@ def create_parser(mach_interface=False):
             help='Specify the url for the repository we are testing. '
                  'This will use the value found in application.ini if'
                  ' it is not specified.')
+    add_arg('--framework',
+            help='Will post to the specified framework for Perfherder. '
+                 'Default "talos".  Used primarily for experiments on '
+                 'new platforms')
     add_arg('--print-tests', action=_ListTests,
             help="print available tests")
     add_arg('--print-suites', action=_ListSuite,
