@@ -94,8 +94,9 @@ DEFAULTS = dict(
         'take_over_this_computer': True,
         'browser.newtabpage.directory.source':
             '${webserver}/directoryLinks.json',
-        'browser.newtabpage.directory.ping': '',
         'browser.newtabpage.introShown': True,
+        'browser.safebrowsing.downloads.remote.url':
+            'http://127.0.0.1/safebrowsing-dummy/downloads',
         'browser.safebrowsing.provider.google.gethashURL':
             'http://127.0.0.1/safebrowsing-dummy/gethash',
         'browser.safebrowsing.provider.google.updateURL':
@@ -112,14 +113,12 @@ DEFAULTS = dict(
             'http://127.0.0.1/trackingprotection/tour',
         'browser.safebrowsing.phishing.enabled': False,
         'browser.safebrowsing.malware.enabled': False,
-        'browser.safebrowsing.forbiddenURIs.enabled': False,
         'browser.safebrowsing.blockedURIs.enabled': False,
         'privacy.trackingprotection.enabled': False,
         'privacy.trackingprotection.pbmode.enabled': False,
         'browser.search.isUS': True,
         'browser.search.countryCode': 'US',
-        'browser.selfsupport.url':
-            'https://127.0.0.1/selfsupport-dummy/',
+        'browser.urlbar.userMadeSearchSuggestionsChoice': True,
         'extensions.update.url':
             'http://127.0.0.1/extensions-dummy/updateURL',
         'extensions.update.background.url':
@@ -174,16 +173,15 @@ DEFAULTS = dict(
         'experiments.manifest.uri':
             'https://127.0.0.1/experiments-dummy/manifest',
         'network.http.speculative-parallel-limit': 0,
-        'app.update.badge': False,
         'lightweightThemes.selectedThemeID': "",
-        'devtools.webide.widget.enabled': False,
-        'devtools.webide.widget.inNavbarByDefault': False,
         'devtools.chrome.enabled': False,
         'devtools.debugger.remote-enabled': False,
         'devtools.theme': "light",
         'devtools.timeline.enabled': False,
         'identity.fxaccounts.migrateToDevEdition': False,
-        'media.libavcodec.allow-obsolete': True
+        'plugin.state.flash': 0,
+        'media.libavcodec.allow-obsolete': True,
+        'extensions.legacy.enabled': True
     }
 )
 
