@@ -15,6 +15,7 @@ import mozhttpd
 
 from mozlog import get_proxy_logger
 
+from talos.mitmproxy import mitmproxy
 from talos.results import TalosResults
 from talos.ttest import TTest
 from talos.utils import TalosError, TalosRegression
@@ -47,6 +48,14 @@ def buildCommandLine(test):
     if 'tpmanifest' not in test:
         raise TalosError("tpmanifest not found in test: %s" % test)
 
+    # if profiling is on, override tppagecycles to prevent test hanging
+    if test['gecko_profile']:
+        LOG.info("Gecko profiling is enabled so talos is reducing the number "
+                 "of cycles, please disregard reported numbers")
+        for cycle_var in ['tppagecycles', 'tpcycles', 'cycles']:
+            if test[cycle_var] > 2:
+                test[cycle_var] = 2
+
     # build pageloader command from options
     url = ['-tp', test['tpmanifest']]
     CLI_bool_options = ['tpchrome', 'tpmozafterpaint', 'tpdisable_e10s',
@@ -81,7 +90,6 @@ def run_tests(config, browser_config):
     # get the test data
     tests = config['tests']
     tests = useBaseTestDefaults(config.get('basetest', {}), tests)
-
     paths = ['profile_path', 'tpmanifest', 'extensions', 'setup', 'cleanup']
     for test in tests:
 
@@ -108,6 +116,12 @@ def run_tests(config, browser_config):
     if browser_config['develop']:
         browser_config['extra_args'] = '--no-remote'
 
+    # with addon signing for production talos, we want to develop without it
+    if browser_config['develop'] or browser_config['branch_name'] == 'Try':
+        browser_config['preferences']['xpinstall.signatures.required'] = False
+
+    browser_config['preferences']['extensions.allow-non-mpc-extensions'] = True
+
     # set defaults
     testdate = config.get('testdate', '')
 
@@ -138,7 +152,7 @@ def run_tests(config, browser_config):
         browser_config['sourcestamp'] = version_info['application_changeset']
     except KeyError:
         if not browser_config['develop']:
-            print "unable to find changeset or repository: %s" % version_info
+            print("unable to find changeset or repository: %s" % version_info)
             sys.exit()
         else:
             browser_config['repository'] = 'develop'
@@ -157,20 +171,57 @@ def run_tests(config, browser_config):
     talos_results = TalosResults()
 
     # results links
-    if not browser_config['develop']:
+    if not browser_config['develop'] and not config['gecko_profile']:
         results_urls = dict(
             # another hack; datazilla stands for Perfherder
             # and do not require url, but a non empty dict is required...
-            datazilla_urls=['local.json'],
+            output_urls=['local.json'],
         )
     else:
         # local mode, output to files
-        results_urls = dict(datazilla_urls=[os.path.abspath('local.json')])
-    talos_results.check_output_formats(results_urls)
+        results_urls = dict(output_urls=[os.path.abspath('local.json')])
 
     httpd = setup_webserver(browser_config['webserver'])
     httpd.start()
 
+    # if e10s add as extra results option
+    if config['e10s']:
+        talos_results.add_extra_option('e10s')
+
+    if config['gecko_profile']:
+        talos_results.add_extra_option('geckoProfile')
+
+    # some tests use mitmproxy to playback pages
+    mitmproxy_recordings_list = config.get('mitmproxy', False)
+    if mitmproxy_recordings_list is not False:
+        # needed so can tell talos ttest to allow external connections
+        browser_config['mitmproxy'] = True
+
+        # start mitmproxy playback; this also generates the CA certificate
+        mitmdump_path = config.get('mitmdumpPath', False)
+        if mitmdump_path is False:
+            # cannot continue, need path for mitmdump playback tool
+            LOG.error('Aborting: mitmdumpPath was not provided on cmd line but is required')
+            sys.exit()
+
+        mitmproxy_recording_path = os.path.join(here, 'mitmproxy')
+        mitmproxy_proc = mitmproxy.start_mitmproxy_playback(mitmdump_path,
+                                                            mitmproxy_recording_path,
+                                                            mitmproxy_recordings_list.split(),
+                                                            browser_config['browser_path'])
+
+        # install the generated CA certificate into Firefox
+        # mitmproxy cert setup needs path to mozharness install; mozharness has set this
+        # value in the SCRIPTSPATH env var for us in mozharness/mozilla/testing/talos.py
+        scripts_path = os.environ.get('SCRIPTSPATH')
+        LOG.info('scripts_path: %s' % str(scripts_path))
+        mitmproxy.install_mitmproxy_cert(mitmproxy_proc,
+                                         browser_config['browser_path'],
+                                         str(scripts_path))
+
+    if config.get('first_non_blank_paint', False):
+        browser_config['firstNonBlankPaint'] = True
+
     testname = None
     # run the tests
     timer = utils.Timer()
@@ -189,14 +240,14 @@ def run_tests(config, browser_config):
         LOG.error("Detected a regression for %s" % testname)
         # by returning 1, we report an orange to buildbot
         # http://docs.buildbot.net/latest/developer/results.html
-        LOG.test_end(testname, status='FAIL', message=unicode(exc),
+        LOG.test_end(testname, status='FAIL', message=str(exc),
                      stack=traceback.format_exc())
         return 1
     except Exception as exc:
         # NOTE: if we get into this condition, talos has an internal
         # problem and cannot continue
         #       this will prevent future tests from running
-        LOG.test_end(testname, status='ERROR', message=unicode(exc),
+        LOG.test_end(testname, status='ERROR', message=str(exc),
                      stack=traceback.format_exc())
         # indicate a failure to buildbot, turn the job red
         return 2
@@ -206,13 +257,16 @@ def run_tests(config, browser_config):
 
     LOG.info("Completed test suite (%s)" % timer.elapsed())
 
+    # if mitmproxy was used for page playback, stop it
+    if mitmproxy_recordings_list is not False:
+        mitmproxy.stop_mitmproxy_playback(mitmproxy_proc)
+
     # output results
     if results_urls:
         talos_results.output(results_urls)
-        if browser_config['develop']:
-            print
-            print ("Thanks for running Talos locally. Results are in"
-                   " %s and %s" % (results_urls['datazilla_urls']))
+        if browser_config['develop'] or config['gecko_profile']:
+            print("Thanks for running Talos locally. Results are in %s"
+                  % (results_urls['output_urls']))
 
     # we will stop running tests on a failed test, or we will return 0 for
     # green
@@ -222,7 +276,7 @@ def run_tests(config, browser_config):
 def main(args=sys.argv[1:]):
     try:
         config, browser_config = get_configs()
-    except ConfigurationError, exc:
+    except ConfigurationError as exc:
         sys.exit("ERROR: %s" % exc)
     sys.exit(run_tests(config, browser_config))
 
