@@ -94,10 +94,10 @@ class TsBase(Test):
                          # initialization of the test. If some of the files may
                          # be overwritten by Firefox and need to be reinstalled
                          # before each pass, use key |reinstall|
-        'sps_profile',
-        'sps_profile_interval',
-        'sps_profile_entries',
-        'sps_profile_startup',
+        'gecko_profile',
+        'gecko_profile_interval',
+        'gecko_profile_entries',
+        'gecko_profile_startup',
         'preferences',
         'xperf_counters',
         'xperf_providers',
@@ -108,6 +108,7 @@ class TsBase(Test):
         'filters',
         'setup',
         'cleanup',
+        'webextensions',
         'reinstall',     # A list of files from the profile directory that
                          # should be copied to the temporary profile prior to
                          # running each cycle, to avoid one cycle overwriting
@@ -127,8 +128,8 @@ class ts_paint(TsBase):
     """
     cycles = 20
     timeout = 150
-    sps_profile_startup = True
-    sps_profile_entries = 10000000
+    gecko_profile_startup = True
+    gecko_profile_entries = 10000000
     url = 'startup_test/tspaint_test.html'
     shutdown = False
     xperf_counters = []
@@ -141,6 +142,12 @@ class ts_paint(TsBase):
     unit = 'ms'
 
 
+@register_test()
+class ts_paint_webext(ts_paint):
+    webextensions = '${talos}/webextensions/dummy/dummy-signed.xpi'
+    preferences = {'xpinstall.signatures.required': False}
+
+
 @register_test()
 class sessionrestore(TsBase):
     """
@@ -153,9 +160,9 @@ class sessionrestore(TsBase):
     extensions = \
         '${talos}/startup_test/sessionrestore/addon/sessionrestore-signed.xpi'
     cycles = 10
-    timeout = 1000000
-    sps_profile_startup = True
-    sps_profile_entries = 10000000
+    timeout = 900
+    gecko_profile_startup = True
+    gecko_profile_entries = 10000000
     profile_path = '${talos}/startup_test/sessionrestore/profile'
     url = 'startup_test/sessionrestore/index.html'
     shutdown = False
@@ -189,8 +196,8 @@ class tpaint(TsBase):
     """
     url = 'file://${talos}/startup_test/tpaint.html?auto=1'
     timeout = 300
-    sps_profile_interval = 1
-    sps_profile_entries = 2000000
+    gecko_profile_interval = 1
+    gecko_profile_entries = 2000000
     tpmozafterpaint = True
     filters = filter.ignore_first.prepare(5) + filter.median.prepare()
     unit = 'ms'
@@ -205,8 +212,8 @@ class tresize(TsBase):
     cycles = 20
     url = 'startup_test/tresize/addon/content/tresize-test.html'
     timeout = 150
-    sps_profile_interval = 2
-    sps_profile_entries = 1000000
+    gecko_profile_interval = 2
+    gecko_profile_entries = 1000000
     tpmozafterpaint = True
     filters = filter.ignore_first.prepare(5) + filter.median.prepare()
     unit = 'ms'
@@ -229,13 +236,26 @@ class PageloaderTest(Test):
     timeout = None
     keys = ['tpmanifest', 'tpcycles', 'tppagecycles', 'tprender', 'tpchrome',
             'tpmozafterpaint', 'tploadnocache', 'rss', 'mainthread',
-            'resolution', 'cycles', 'sps_profile', 'sps_profile_interval',
-            'sps_profile_entries', 'tptimeout', 'win_counters', 'w7_counters',
+            'resolution', 'cycles', 'gecko_profile', 'gecko_profile_interval',
+            'gecko_profile_entries', 'tptimeout', 'win_counters', 'w7_counters',
             'linux_counters', 'mac_counters', 'tpscrolltest', 'xperf_counters',
             'timeout', 'shutdown', 'responsiveness', 'profile_path',
             'xperf_providers', 'xperf_user_providers', 'xperf_stackwalk',
             'filters', 'preferences', 'extensions', 'setup', 'cleanup',
-            'lower_is_better', 'alert_threshold', 'unit']
+            'lower_is_better', 'alert_threshold', 'unit', 'webextensions']
+
+
+class QuantumPageloadTest(PageloaderTest):
+    """
+    Base class for a Quantum Pageload test
+    """
+    tpcycles = 1
+    tppagecycles = 25
+    gecko_profile_interval = 1
+    gecko_profile_entries = 2000000
+    filters = filter.ignore_first.prepare(5) + filter.median.prepare()
+    unit = 'ms'
+    lower_is_better = True
 
 
 @register_test()
@@ -247,7 +267,7 @@ class tabpaint(PageloaderTest):
     extensions = '${talos}/tests/tabpaint/tabpaint-signed.xpi'
     tpmanifest = '${talos}/tests/tabpaint/tabpaint.manifest'
     tppagecycles = 20
-    sps_profile_entries = 1000000
+    gecko_profile_entries = 1000000
     tploadnocache = True
     unit = 'ms'
     preferences = {
@@ -269,7 +289,7 @@ class tps(PageloaderTest):
     extensions = '${talos}/tests/tabswitch/tabswitch-signed.xpi'
     tpmanifest = '${talos}/tests/tabswitch/tps.manifest'
     tppagecycles = 5
-    sps_profile_entries = 1000000
+    gecko_profile_entries = 1000000
     tploadnocache = True
     preferences = {
         'addon.test.tabswitch.urlfile': os.path.join('${talos}',
@@ -313,8 +333,8 @@ class tart(PageloaderTest):
     tppagecycles = 25
     tploadnocache = True
     tpmozafterpaint = False
-    sps_profile_interval = 10
-    sps_profile_entries = 1000000
+    gecko_profile_interval = 10
+    gecko_profile_entries = 1000000
     win_counters = w7_counters = linux_counters = mac_counters = None
     """
     ASAP mode
@@ -350,8 +370,8 @@ class cart(PageloaderTest):
     tppagecycles = 25
     tploadnocache = True
     tpmozafterpaint = False
-    sps_profile_interval = 1
-    sps_profile_entries = 10000000
+    gecko_profile_interval = 1
+    gecko_profile_entries = 10000000
     win_counters = w7_counters = linux_counters = mac_counters = None
     """
     ASAP mode
@@ -376,8 +396,8 @@ class damp(PageloaderTest):
     tppagecycles = 25
     tploadnocache = True
     tpmozafterpaint = False
-    sps_profile_interval = 10
-    sps_profile_entries = 1000000
+    gecko_profile_interval = 10
+    gecko_profile_entries = 1000000
     win_counters = w7_counters = linux_counters = mac_counters = None
     filters = filter.ignore_first.prepare(1) + filter.median.prepare()
     preferences = {'devtools.memory.enabled': True,
@@ -400,8 +420,8 @@ class glterrain(PageloaderTest):
     tppagecycles = 25
     tploadnocache = True
     tpmozafterpaint = False
-    sps_profile_interval = 10
-    sps_profile_entries = 2000000
+    gecko_profile_interval = 10
+    gecko_profile_entries = 2000000
     win_counters = w7_counters = linux_counters = mac_counters = None
     """ ASAP mode """
     preferences = {'layout.frame_rate': 0,
@@ -411,6 +431,25 @@ class glterrain(PageloaderTest):
     unit = 'frame interval'
 
 
+@register_test()
+class glvideo(PageloaderTest):
+    """
+    WebGL video texture update with 1080p video.
+    Measures mean tick time across 100 ticks.
+    (each tick is texImage2D(<video>)+setTimeout(0))
+    """
+    tpmanifest = '${talos}/tests/webgl/glvideo.manifest'
+    tpcycles = 1
+    tppagecycles = 5
+    tploadnocache = True
+    tpmozafterpaint = False
+    gecko_profile_interval = 2
+    gecko_profile_entries = 2000000
+    win_counters = w7_counters = linux_counters = mac_counters = None
+    filters = filter.ignore_first.prepare(1) + filter.median.prepare()
+    unit = 'ms'
+
+
 @register_test()
 class tp5n(PageloaderTest):
     """
@@ -475,13 +514,19 @@ class tp5o(PageloaderTest):
     linux_counters = ['Private Bytes', 'XRes', 'Main_RSS']
     mac_counters = ['Main_RSS']
     responsiveness = True
-    sps_profile_interval = 2
-    sps_profile_entries = 4000000
+    gecko_profile_interval = 2
+    gecko_profile_entries = 4000000
     filters = filter.ignore_first.prepare(5) + filter.median.prepare()
     timeout = 1800
     unit = 'ms'
 
 
+@register_test()
+class tp5o_webext(tp5o):
+    webextensions = '${talos}/webextensions/dummy/dummy-signed.xpi'
+    preferences = {'xpinstall.signatures.required': False}
+
+
 @register_test()
 class tp5o_scroll(PageloaderTest):
     """
@@ -490,8 +535,8 @@ class tp5o_scroll(PageloaderTest):
     tpmanifest = '${talos}/tests/tp5n/tp5o.manifest'
     tpcycles = 1
     tppagecycles = 12
-    sps_profile_interval = 2
-    sps_profile_entries = 2000000
+    gecko_profile_interval = 2
+    gecko_profile_entries = 2000000
 
     tpscrolltest = True
     """ASAP mode"""
@@ -515,8 +560,8 @@ class v8_7(PageloaderTest):
     selective branches and operating systems.
     """
     tpmanifest = '${talos}/tests/v8_7/v8.manifest'
-    sps_profile_interval = 1
-    sps_profile_entries = 1000000
+    gecko_profile_interval = 1
+    gecko_profile_entries = 1000000
     tpcycles = 1
     resolution = 20
     tpmozafterpaint = False
@@ -535,8 +580,8 @@ class kraken(PageloaderTest):
     tpmanifest = '${talos}/tests/kraken/kraken.manifest'
     tpcycles = 1
     tppagecycles = 1
-    sps_profile_interval = 0.1
-    sps_profile_entries = 1000000
+    gecko_profile_interval = 0.1
+    gecko_profile_entries = 1000000
     tpmozafterpaint = False
     preferences = {'dom.send_after_paint_to_content': False}
     filters = filter.mean.prepare()
@@ -552,8 +597,8 @@ class basic_compositor_video(PageloaderTest):
     tpcycles = 1
     tppagecycles = 12
     timeout = 10000
-    sps_profile_interval = 1
-    sps_profile_entries = 2000000
+    gecko_profile_interval = 1
+    gecko_profile_entries = 2000000
     preferences = {'full-screen-api.allow-trusted-requests-only': False,
                    'layers.acceleration.force-enabled': False,
                    'layers.acceleration.disabled': True,
@@ -576,8 +621,8 @@ class tcanvasmark(PageloaderTest):
     tpcycles = 5
     tppagecycles = 1
     timeout = 900
-    sps_profile_interval = 10
-    sps_profile_entries = 2500000
+    gecko_profile_interval = 10
+    gecko_profile_entries = 2500000
     tpmozafterpaint = False
     preferences = {'dom.send_after_paint_to_content': False}
     filters = filter.ignore_first.prepare(1) + filter.median.prepare()
@@ -601,8 +646,8 @@ class dromaeo_css(dromaeo):
 
     Each page in the manifest is part of the dromaemo css benchmark.
     """
-    sps_profile_interval = 2
-    sps_profile_entries = 10000000
+    gecko_profile_interval = 2
+    gecko_profile_entries = 10000000
     tpmanifest = '${talos}/tests/dromaeo/css.manifest'
     unit = 'score'
 
@@ -616,8 +661,8 @@ class dromaeo_dom(dromaeo):
 
     Each page in the manifest is part of the dromaemo dom benchmark.
     """
-    sps_profile_interval = 2
-    sps_profile_entries = 10000000
+    gecko_profile_interval = 2
+    gecko_profile_entries = 10000000
     tpmanifest = '${talos}/tests/dromaeo/dom.manifest'
     tpdisable_e10s = True
     unit = 'score'
@@ -633,8 +678,8 @@ class tsvgm(PageloaderTest):
     tpcycles = 1
     tppagecycles = 7
     tpmozafterpaint = False
-    sps_profile_interval = 10
-    sps_profile_entries = 1000000
+    gecko_profile_interval = 10
+    gecko_profile_entries = 1000000
     """ASAP mode"""
     preferences = {'layout.frame_rate': 0,
                    'docshell.event_starvation_delay_hint': 1,
@@ -653,8 +698,8 @@ class tsvgx(PageloaderTest):
     tpcycles = 1
     tppagecycles = 25
     tpmozafterpaint = False
-    sps_profile_interval = 10
-    sps_profile_entries = 1000000
+    gecko_profile_interval = 10
+    gecko_profile_entries = 1000000
     """ASAP mode"""
     preferences = {'layout.frame_rate': 0,
                    'docshell.event_starvation_delay_hint': 1,
@@ -673,8 +718,8 @@ class tsvg_static(PageloaderTest):
     tpcycles = 1
     tppagecycles = 25
     tpmozafterpaint = True
-    sps_profile_interval = 1
-    sps_profile_entries = 10000000
+    gecko_profile_interval = 1
+    gecko_profile_entries = 10000000
     filters = filter.ignore_first.prepare(5) + filter.median.prepare()
     unit = 'ms'
 
@@ -688,8 +733,8 @@ class tsvgr_opacity(PageloaderTest):
     tpcycles = 1
     tppagecycles = 25
     tpmozafterpaint = True
-    sps_profile_interval = 1
-    sps_profile_entries = 10000000
+    gecko_profile_interval = 1
+    gecko_profile_entries = 10000000
     filters = filter.ignore_first.prepare(5) + filter.median.prepare()
     unit = 'ms'
 
@@ -703,8 +748,8 @@ class tscrollx(PageloaderTest):
     tpcycles = 1
     tppagecycles = 25
     tpmozafterpaint = False
-    sps_profile_interval = 1
-    sps_profile_entries = 1000000
+    gecko_profile_interval = 1
+    gecko_profile_entries = 1000000
     """ ASAP mode """
     preferences = {'layout.frame_rate': 0,
                    'docshell.event_starvation_delay_hint': 1,
@@ -728,3 +773,67 @@ class a11yr(PageloaderTest):
     preferences = {'dom.send_after_paint_to_content': False}
     unit = 'ms'
     alert_threshold = 5.0
+
+
+@register_test()
+class bloom_basic(PageloaderTest):
+    """
+    Stylo bloom_basic test
+    """
+    tpmanifest = '${talos}/tests/perf-reftest/bloom_basic.manifest'
+    tpcycles = 1
+    tppagecycles = 25
+    gecko_profile_interval = 1
+    gecko_profile_entries = 2000000
+    filters = filter.ignore_first.prepare(5) + filter.median.prepare()
+    unit = 'ms'
+    lower_is_better = True
+    alert_threshold = 5.0
+
+
+@register_test()
+class bloom_basic_ref(PageloaderTest):
+    """
+    Stylo bloom_basic_ref test
+    """
+    tpmanifest = '${talos}/tests/perf-reftest/bloom_basic_ref.manifest'
+    tpcycles = 1
+    tppagecycles = 25
+    gecko_profile_interval = 1
+    gecko_profile_entries = 2000000
+    filters = filter.ignore_first.prepare(5) + filter.median.prepare()
+    unit = 'ms'
+    lower_is_better = True
+    alert_threshold = 5.0
+
+
+@register_test()
+class quantum_pageload_google(QuantumPageloadTest):
+    """
+    Quantum Pageload Test - Google
+    """
+    tpmanifest = '${talos}/tests/quantum_pageload/quantum_pageload_google.manifest'
+
+
+@register_test()
+class quantum_pageload_youtube(QuantumPageloadTest):
+    """
+    Quantum Pageload Test - YouTube
+    """
+    tpmanifest = '${talos}/tests/quantum_pageload/quantum_pageload_youtube.manifest'
+
+
+@register_test()
+class quantum_pageload_amazon(QuantumPageloadTest):
+    """
+    Quantum Pageload Test - Amazon
+    """
+    tpmanifest = '${talos}/tests/quantum_pageload/quantum_pageload_amazon.manifest'
+
+
+@register_test()
+class quantum_pageload_facebook(QuantumPageloadTest):
+    """
+    Quantum Pageload Test - Facebook
+    """
+    tpmanifest = '${talos}/tests/quantum_pageload/quantum_pageload_facebook.manifest'
