@@ -60,6 +60,12 @@ class TTest(object):
         """
 
         LOG.debug("operating with platform_type : %s" % self.platform_type)
+
+        # Bug 1262954: winxp + e10s, disable hwaccel
+        if self.platform_type == "win_" and browser_config['e10s']:
+            prefs = browser_config['preferences']
+            prefs['layers.acceleration.disabled'] = True
+
         with FFSetup(browser_config, test_config) as setup:
             return self._runTest(browser_config, test_config, setup)
 
@@ -91,18 +97,26 @@ class TTest(object):
         if test_config['shutdown']:
             global_counters['shutdown'] = []
         if test_config.get('responsiveness') and \
-                platform.system() != "Linux":
-            # ignore responsiveness tests on linux until we fix
-            # Bug 710296
+           platform.system() != "Darwin":
+            # ignore osx for now as per bug 1245793
             setup.env['MOZ_INSTRUMENT_EVENT_LOOP'] = '1'
             setup.env['MOZ_INSTRUMENT_EVENT_LOOP_THRESHOLD'] = '20'
             setup.env['MOZ_INSTRUMENT_EVENT_LOOP_INTERVAL'] = '10'
             global_counters['responsiveness'] = []
 
+        setup.env['JSGC_DISABLE_POISONING'] = '1'
+        setup.env['MOZ_DISABLE_NONLOCAL_CONNECTIONS'] = '1'
+
+        # if using mitmproxy we must allow access to 'external' sites
+        if browser_config.get('mitmproxy', False):
+            LOG.info("Using mitmproxy so setting MOZ_DISABLE_NONLOCAL_CONNECTIONS to 0")
+            setup.env['MOZ_DISABLE_NONLOCAL_CONNECTIONS'] = '0'
+
         # instantiate an object to hold test results
         test_results = results.TestResults(
             test_config,
-            global_counters
+            global_counters,
+            browser_config.get('framework')
         )
 
         for i in range(test_config['cycles']):
@@ -125,7 +139,7 @@ class TTest(object):
 
             # Run the test
             timeout = test_config.get('timeout', 7200)  # 2 hours default
-            if setup.sps_profile:
+            if setup.gecko_profile:
                 # When profiling, give the browser some extra time
                 # to dump the profile.
                 timeout += 5 * 60
@@ -135,8 +149,8 @@ class TTest(object):
                 browser_config["extra_args"],
                 setup.profile_dir,
                 test_config['url'],
-                profiling_info=(setup.sps_profile.profiling_info
-                                if setup.sps_profile else None)
+                profiling_info=(setup.gecko_profile.profiling_info
+                                if setup.gecko_profile else None)
             )
 
             mainthread_error_count = 0
@@ -167,6 +181,10 @@ class TTest(object):
                     on_started=(counter_management.start
                                 if counter_management else None),
                 )
+            except:
+                self.check_for_crashes(browser_config, minidump_dir,
+                                       test_config['name'])
+                raise
             finally:
                 if counter_management:
                     counter_management.stop()
@@ -189,7 +207,7 @@ class TTest(object):
                         if line.strip() == "":
                             continue
 
-                        print line
+                        print(line)
                         mainthread_error_count += 1
                     mozfile.remove(rawlog)
 
@@ -226,8 +244,8 @@ class TTest(object):
                                  else None)
             )
 
-            if setup.sps_profile:
-                setup.sps_profile.symbolicate(i)
+            if setup.gecko_profile:
+                setup.gecko_profile.symbolicate(i)
 
             self.check_for_crashes(browser_config, minidump_dir,
                                    test_config['name'])
