@@ -254,7 +254,12 @@ class AutoTry(object):
             'action': 'store_true',
             'dest': 'artifact',
             'help': 'Force artifact builds where possible.',
-        }
+        },
+        '--upload-xdbs': {
+            'action': 'store_true',
+            'dest': 'upload_xdbs',
+            'help': 'Upload XDB compilation db files generated by hazard build',
+        },
     }
 
     def __init__(self, topsrcdir, resolver_func, mach_context):
