@@ -12,10 +12,19 @@ var metadataElements = {
     crossOrigin: {type: "enum", keywords: ["anonymous", "use-credentials"], nonCanon:{"": "anonymous"}, isNullable: true, defaultVal: null, invalidVal: "anonymous"},
     rel: "string",
     relList: {type: "tokenlist", domAttrName: "rel"},
+    as: {
+      type: "enum",
+      keywords: ["", "audio", "document", "embed", "font", "image", "manifest", "object", "report", "script", "serviceworker", "sharedworker", "style", "track", "video", "worker", "xslt"],
+      defaultVal: "",
+      invalidVal: ""
+    },
     media: "string",
+    nonce: "string",
+    integrity: "string",
     hreflang: "string",
     type: "string",
     sizes: "settable tokenlist",
+    referrerPolicy: {type: "enum", keywords: ["", "no-referrer", "no-referrer-when-downgrade", "same-origin", "origin", "strict-origin", "origin-when-cross-origin", "strict-origin-when-cross-origin", "unsafe-url"]},
 
     // Obsolete
     charset: "string",
