@@ -152,7 +152,6 @@ function VisualOutput(elem, runner) {
     this.section_wrapper = null;
     this.results_table = this.elem.querySelector(".results > table");
     this.section = null;
-    this.manifest_status = this.elem.querySelector("#manifest");
     this.progress = this.elem.querySelector(".summary .progress");
     this.meter = this.progress.querySelector(".progress-bar");
     this.result_count = null;
@@ -195,7 +194,7 @@ VisualOutput.prototype = {
         }
         this.meter.style.width = '0px';
         this.meter.textContent = '0%';
-        this.manifest_status.style.display = "none";
+        this.meter.classList.remove("stopped", "loading-manifest");
         this.elem.querySelector(".jsonResults").style.display = "none";
         this.results_table.removeChild(this.results_table.tBodies[0]);
         this.results_table.appendChild(document.createElement("tbody"));
@@ -205,14 +204,13 @@ VisualOutput.prototype = {
         this.clear();
         this.instructions.style.display = "none";
         this.elem.style.display = "block";
-        this.manifest_status.style.display = "inline";
+        this.steady_status("loading-manifest");
     },
 
     on_start: function() {
         this.clear();
         this.instructions.style.display = "none";
         this.elem.style.display = "block";
-        this.meter.classList.remove("stopped");
         this.meter.classList.add("progress-striped", "active");
     },
 
@@ -281,17 +279,24 @@ VisualOutput.prototype = {
         this.update_meter(this.runner.progress(), this.runner.results.count(), this.runner.test_count());
     },
 
-    on_done: function() {
+    steady_status: function(statusName) {
+        var statusTexts = {
+            done: "Done!",
+            stopped: "Stopped",
+            "loading-manifest": "Updating and loading test manifest; this may take several minutes."
+        };
+        var textContent = statusTexts[statusName];
+
         this.meter.setAttribute("aria-valuenow", this.meter.getAttribute("aria-valuemax"));
         this.meter.style.width = "100%";
-        if (this.runner.stop_flag) {
-            this.meter.textContent = "Stopped";
-            this.meter.classList.add("stopped");
-        } else {
-            this.meter.textContent = "Done!";
-        }
-        this.meter.classList.remove("progress-striped", "active");
+        this.meter.textContent = textContent;
+        this.meter.classList.remove("progress-striped", "active", "stopped", "loading-manifest");
+        this.meter.classList.add(statusName);
         this.runner.test_div.textContent = "";
+    },
+
+    on_done: function() {
+        this.steady_status(this.runner.stop_flag ? "stopped" : "done");
         //add the json serialization of the results
         var a = this.elem.querySelector(".jsonResults");
         var json = this.runner.results.to_json();
@@ -644,8 +649,10 @@ Runner.prototype = {
         return this.manifest[this.mTestCount];
     },
 
-    open_test_window: function() {
-        this.test_window = window.open("about:blank", 800, 600);
+    ensure_test_window: function() {
+        if (!this.test_window || this.test_window.location === null) {
+          this.test_window = window.open("about:blank", 800, 600);
+        }
     },
 
     manifest_loaded: function() {
@@ -666,6 +673,7 @@ Runner.prototype = {
         this.manifest_iterator = new ManifestIterator(this.manifest, this.path, this.test_types, this.use_regex);
         this.num_tests = null;
 
+        this.ensure_test_window();
         if (this.manifest.data === null) {
             this.wait_for_manifest();
         } else {
@@ -682,7 +690,6 @@ Runner.prototype = {
 
     do_start: function() {
         if (this.manifest_iterator.count() > 0) {
-            this.open_test_window();
             this.start_callbacks.forEach(function(callback) {
                 callback();
             });
@@ -727,6 +734,7 @@ Runner.prototype = {
         this.done_flag = true;
         if (this.test_window) {
             this.test_window.close();
+            this.test_window = undefined;
         }
         this.done_callbacks.forEach(function(callback) {
             callback();
@@ -758,9 +766,7 @@ Runner.prototype = {
     },
 
     load: function(path) {
-        if (this.test_window.location === null) {
-            this.open_test_window();
-        }
+        this.ensure_test_window();
         this.test_window.location.href = this.server + path;
     },
 
