@@ -566,8 +566,6 @@ tail =
         self.assertTrue(any(re.search(line_pat, line) for line in log_lines),
                         "No line resembling a stack frame was found in\n%s" % pprint.pformat(log_lines))
 
-    @unittest.skipIf(mozinfo.info.get('stylo'),
-                     'failing on stylo for some reason') # bug 1337667
     def testChildPass(self):
         """
         Check that a simple test running in a child process passes.
@@ -586,9 +584,6 @@ tail =
         self.assertInLog("CHILD-TEST-COMPLETED")
         self.assertNotInLog(TEST_FAIL_STRING)
 
-
-    @unittest.skipIf(mozinfo.info.get('stylo'),
-                     'failing on stylo for some reason') # bug 1337667
     def testChildFail(self):
         """
         Check that a simple failing test running in a child process fails.
@@ -607,8 +602,6 @@ tail =
         self.assertInLog("CHILD-TEST-COMPLETED")
         self.assertNotInLog(TEST_PASS_STRING)
 
-    @unittest.skipIf(mozinfo.info.get('stylo'),
-                     'failing on stylo for some reason') # bug 1337667
     def testChildHang(self):
         """
         Check that incomplete output from a child process results in a
@@ -628,8 +621,6 @@ tail =
         self.assertNotInLog("CHILD-TEST-COMPLETED")
         self.assertNotInLog(TEST_PASS_STRING)
 
-    @unittest.skipIf(mozinfo.info.get('stylo'),
-                     'failing on stylo for some reason') # bug 1337667
     def testChild(self):
         """
         Checks that calling do_load_child_test_harness without run_test_in_child
@@ -1325,8 +1316,6 @@ add_test({
         self.assertInLog(TEST_PASS_STRING)
         self.assertNotInLog(TEST_FAIL_STRING)
 
-    @unittest.skipIf(mozinfo.info.get('stylo'),
-                     'failing on stylo for some reason') # bug 1337667
     def testChildMozinfo(self):
         """
         Check that mozinfo.json is loaded in child process
